@@ -2,10 +2,12 @@
 # -*- coding: utf-8 -*-
 
 # dimension.py
-# definitons of dimension characters
+# definitions of dimension characters
 
 import math
+from distutils.version import LooseVersion
 
+import geopandas as gpd
 import numpy as np
 import pandas as pd
 import scipy as sp
@@ -14,6 +16,8 @@ from tqdm import tqdm
 
 from .shape import _make_circle
 
+GPD_08 = str(gpd.__version__) >= LooseVersion("0.8.0")
+
 __all__ = [
     "Area",
     "Perimeter",
@@ -35,7 +39,7 @@ class Area:
     Calculates area of each object in given GeoDataFrame. It can be used for any
     suitable element (building footprint, plot, tessellation, block).
 
-    It is a simple wrapper for geopandas `gdf.geometry.area` for the consistency of momepy.
+    It is a simple wrapper for GeoPandas ``.area`` for the consistency of momepy.
 
     Parameters
     ----------
@@ -68,7 +72,7 @@ class Perimeter:
     Calculates perimeter of each object in given GeoDataFrame. It can be used for any
     suitable element (building footprint, plot, tessellation, block).
 
-    It is a simple wrapper for geopandas `gdf.geometry.length` for the consistency of momepy.
+    It is a simple wrapper for GeoPandas ``.length`` for the consistency of momepy.
 
     Parameters
     ----------
@@ -108,9 +112,9 @@ class Volume:
     gdf : GeoDataFrame
         GeoDataFrame containing objects to analyse
     heights : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored height value
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored height value
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to None, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -178,9 +182,9 @@ class FloorArea:
     gdf : GeoDataFrame
         GeoDataFrame containing objects to analyse
     heights : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored height value
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored height value
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to None, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -236,14 +240,14 @@ class CourtyardArea:
     """
     Calculates area of holes within geometry - area of courtyards.
 
-    Ensure that your geometry is shapely Polygon.
+    Ensure that your geometry is ``shapely.geometry.Polygon``.
 
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects to analyse
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to None, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -325,22 +329,25 @@ class LongestAxisLength:
 
 class AverageCharacter:
     """
-    Calculates the average of a character within a set neighbourhood defined in spatial_weights
+    Calculates the average of a character within a set neighbourhood defined in ``spatial_weights``
 
-    Average value of the character within a set neighbourhood defined in spatial_weights.
-    Can be set to `mean`, `median` or `mode`. `mean` is defined as:
+    Average value of the character within a set neighbourhood defined in ``spatial_weights``.
+    Can be set to ``mean``, ``median`` or ``mode``. ``mean`` is defined as:
 
     .. math::
         \\frac{1}{n}\\left(\\sum_{i=1}^{n} value_{i}\\right)
 
+    Adapted from :cite:`hausleitner2017`.
+
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing morphological tessellation
     values : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored character value.
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored character value.
     unique_id : str
-        name of the column with unique id used as spatial_weights index.
+        name of the column with unique id used as ``spatial_weights`` index.
     spatial_weights : libpysal.weights
         spatial weights matrix
     rng : Two-element sequence containing floats in range of [0,100], optional
@@ -349,6 +356,8 @@ class AverageCharacter:
     mode : str (default 'all')
         mode of average calculation. Can be set to `all`, `mean`, `median` or `mode` or
         list of any of the options.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -373,11 +382,6 @@ class AverageCharacter:
     modes : str
         mode
 
-    References
-    ----------
-    Hausleitner B and Berghauser Pont M (2017) Development of a configurational
-    typology for micro-businesses integrating geometric and configurational variables.
-    [adapted]
 
     Examples
     --------
@@ -388,7 +392,16 @@ class AverageCharacter:
     4823.1334436678835
     """
 
-    def __init__(self, gdf, values, spatial_weights, unique_id, rng=None, mode="all"):
+    def __init__(
+        self,
+        gdf,
+        values,
+        spatial_weights,
+        unique_id,
+        rng=None,
+        mode="all",
+        verbose=True,
+    ):
         self.gdf = gdf
         self.sw = spatial_weights
         self.id = gdf[unique_id]
@@ -424,7 +437,7 @@ class AverageCharacter:
                 raise ValueError("{} is not supported as mode.".format(mode))
             mode = [mode]
 
-        for index in tqdm(data.index, total=data.shape[0]):
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -465,12 +478,11 @@ class StreetProfile:
 
     Returns a dictionary with widths, standard deviation of width, openness, heights,
     standard deviation of height and ratio height/width. Algorithm generates perpendicular
-    lines to `right` dataframe features every `distance` and measures values on intersection
-    with features of `left.` If no feature is reached within
-    `tick_length` its value is set as width (being theoretical maximum).
+    lines to ``right`` dataframe features every ``distance`` and measures values on intersection
+    with features of ``left``. If no feature is reached within
+    ``tick_length`` its value is set as width (being a theoretical maximum).
 
-    .. math::
-        \\
+    Derived from :cite:`araldi2019`.
 
     Parameters
     ----------
@@ -479,12 +491,14 @@ class StreetProfile:
     right : GeoDataFrame
         GeoDataFrame containing buildings along the streets (only Polygon geometry type is supported)
     heights: str, list, np.array, pd.Series (default None)
-        the name of the buildings dataframe column, np.array, or pd.Series where is stored building height. If set to None,
+        the name of the buildings dataframe column, ``np.array``, or ``pd.Series`` where is stored building height. If set to None,
         height and ratio height/width will not be calculated.
     distance : int (default 10)
         distance between perpendicular ticks
     tick_length : int (default 50)
-        lenght of ticks
+        length of ticks
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -507,19 +521,10 @@ class StreetProfile:
     distance : int
         distance between perpendicular ticks
     tick_length : int
-        lenght of ticks
+        length of ticks
     heights : GeoDataFrame
         Series containing used height values
 
-    References
-    ----------
-    Oliveira V (2013) Morpho: a methodology for assessing urban form. Urban Morphology 17(1): 21–33.
-
-    Araldi A and Fusco G (2017) Decomposing and Recomposing Urban Fabric: The City from the Pedestrian
-    Point of View. In: Gervasi O, Murgante B, Misra S, et al. (eds), Computational Science and Its
-    Applications – ICCSA 2017, Lecture Notes in Computer Science, Cham: Springer International
-    Publishing, pp. 365–376. Available from: http://link.springer.com/10.1007/978-3-319-62407-5.
-
     Examples
     --------
     >>> street_profile = momepy.StreetProfile(streets_df, buildings_df, heights='height')
@@ -528,7 +533,9 @@ class StreetProfile:
     >>> streets_df['deviations'] = street_profile.wd
     """
 
-    def __init__(self, left, right, heights=None, distance=10, tick_length=50):
+    def __init__(
+        self, left, right, heights=None, distance=10, tick_length=50, verbose=True
+    ):
         self.left = left
         self.right = right
         self.distance = distance
@@ -550,25 +557,27 @@ class StreetProfile:
         heights_deviations_list = []
         openness_list = []
 
-        for idx, row in tqdm(left.iterrows(), total=left.shape[0]):
+        for shapely_line in tqdm(
+            left.geometry, total=left.shape[0], disable=not verbose
+        ):
             # list to hold all the point coords
             list_points = []
             # set the current distance to place the point
             current_dist = distance
-            # make shapely MultiLineString object
-            shapely_line = row.geometry
             # get the total length of the line
             line_length = shapely_line.length
             # append the starting coordinate to the list
-            list_points.append(Point(list(shapely_line.coords)[0]))
+            list_points.append(list(shapely_line.coords)[0])
             # https://nathanw.net/2012/08/05/generating-chainage-distance-nodes-in-qgis/
             # while the current cumulative distance is less than the total length of the line
             while current_dist < line_length:
                 # use interpolate and increase the current distance
-                list_points.append(shapely_line.interpolate(current_dist))
+                list_points.append(
+                    list(shapely_line.interpolate(current_dist).coords)[0]
+                )
                 current_dist += distance
             # append end coordinate to the list
-            list_points.append(Point(list(shapely_line.coords)[-1]))
+            list_points.append(list(shapely_line.coords)[-1])
 
             ticks = []
             for num, pt in enumerate(list_points, 1):
@@ -578,8 +587,8 @@ class StreetProfile:
                     line_end_1 = self._getPoint1(pt, angle, tick_length / 2)
                     angle = self._getAngle(line_end_1, pt)
                     line_end_2 = self._getPoint2(line_end_1, angle, tick_length)
-                    tick1 = LineString([(line_end_1.x, line_end_1.y), (pt.x, pt.y)])
-                    tick2 = LineString([(line_end_2.x, line_end_2.y), (pt.x, pt.y)])
+                    tick1 = LineString([line_end_1, pt])
+                    tick2 = LineString([line_end_2, pt])
                     ticks.append([tick1, tick2])
 
                 # everything in between
@@ -590,18 +599,8 @@ class StreetProfile:
                     )
                     angle = self._getAngle(line_end_1, list_points[num])
                     line_end_2 = self._getPoint2(line_end_1, angle, tick_length)
-                    tick1 = LineString(
-                        [
-                            (line_end_1.x, line_end_1.y),
-                            (list_points[num].x, list_points[num].y),
-                        ]
-                    )
-                    tick2 = LineString(
-                        [
-                            (line_end_2.x, line_end_2.y),
-                            (list_points[num].x, list_points[num].y),
-                        ]
-                    )
+                    tick1 = LineString([line_end_1, list_points[num]])
+                    tick2 = LineString([line_end_2, list_points[num]])
                     ticks.append([tick1, tick2])
 
                 # end chainage
@@ -610,61 +609,35 @@ class StreetProfile:
                     line_end_1 = self._getPoint1(pt, angle, tick_length / 2)
                     angle = self._getAngle(line_end_1, pt)
                     line_end_2 = self._getPoint2(line_end_1, angle, tick_length)
-                    tick1 = LineString([(line_end_1.x, line_end_1.y), (pt.x, pt.y)])
-                    tick2 = LineString([(line_end_2.x, line_end_2.y), (pt.x, pt.y)])
+                    tick1 = LineString([line_end_1, pt])
+                    tick2 = LineString([line_end_2, pt])
                     ticks.append([tick1, tick2])
             # widths = []
             m_heights = []
             lefts = []
             rights = []
             for duo in ticks:
-
                 for ix, tick in enumerate(duo):
-                    possible_intersections_index = list(
-                        sindex.intersection(tick.bounds)
-                    )
-                    possible_intersections = right.iloc[possible_intersections_index]
-                    real_intersections = possible_intersections.intersects(tick)
-                    get_height = right.loc[list(real_intersections.index)]
-                    possible_int = get_height.exterior.intersection(tick)
-
-                    if not possible_int.is_empty.all():
-                        true_int = []
-                        for one in list(possible_int.index):
-                            if possible_int[one].type == "Point":
-                                true_int.append(possible_int[one])
-                            elif possible_int[one].type == "MultiPoint":
-                                for p in possible_int[one]:
-                                    true_int.append(p)
-
-                        if len(true_int) > 1:
-                            distances = []
-                            ix = 0
-                            for p in true_int:
-                                dist = p.distance(Point(tick.coords[-1]))
-                                distances.append(dist)
-                                ix = ix + 1
-                            minimal = min(distances)
-                            if ix == 0:
-                                lefts.append(minimal)
-                            else:
-                                rights.append(minimal)
+                    if GPD_08:
+                        int_blg = right.iloc[sindex.query(tick, predicate="intersects")]
+                    else:
+                        possible_intersections_index = list(
+                            sindex.intersection(tick.bounds)
+                        )
+                        possible_intersections = right.iloc[
+                            possible_intersections_index
+                        ]
+                        real_intersections = possible_intersections.intersects(tick)
+                        int_blg = possible_intersections[real_intersections]
+                    if not int_blg.empty:
+                        true_int = int_blg.intersection(tick)
+                        dist = true_int.distance(Point(tick.coords[-1]))
+                        if ix == 0:
+                            lefts.append(dist.min())
                         else:
-                            if ix == 0:
-                                lefts.append(
-                                    true_int[0].distance(Point(tick.coords[-1]))
-                                )
-                            else:
-                                rights.append(
-                                    true_int[0].distance(Point(tick.coords[-1]))
-                                )
+                            rights.append(dist.min())
                         if heights is not None:
-                            indices = {}
-                            for idx, row in get_height.iterrows():
-                                dist = row.geometry.distance(Point(tick.coords[-1]))
-                                indices[idx] = dist
-                            minim = min(indices, key=indices.get)
-                            m_heights.append(right.loc[minim][heights])
+                            m_heights.append(int_blg.loc[dist.idxmin()][heights])
 
             openness = (len(lefts) + len(rights)) / len(ticks * 2)
             openness_list.append(1 - openness)
@@ -702,49 +675,62 @@ class StreetProfile:
     # https://glenbambrick.com/tag/perpendicular/
     # angle between two points
     def _getAngle(self, pt1, pt2):
-        x_diff = pt2.x - pt1.x
-        y_diff = pt2.y - pt1.y
+        """
+        pt1, pt2 : tuple
+        """
+        x_diff = pt2[0] - pt1[0]
+        y_diff = pt2[1] - pt1[1]
         return math.degrees(math.atan2(y_diff, x_diff))
 
     # start and end points of chainage tick
     # get the first end point of a tick
     def _getPoint1(self, pt, bearing, dist):
+        """
+        pt : tuple
+        """
         angle = bearing + 90
         bearing = math.radians(angle)
-        x = pt.x + dist * math.cos(bearing)
-        y = pt.y + dist * math.sin(bearing)
-        return Point(x, y)
+        x = pt[0] + dist * math.cos(bearing)
+        y = pt[1] + dist * math.sin(bearing)
+        return (x, y)
 
     # get the second end point of a tick
     def _getPoint2(self, pt, bearing, dist):
+        """
+        pt : tuple
+        """
         bearing = math.radians(bearing)
-        x = pt.x + dist * math.cos(bearing)
-        y = pt.y + dist * math.sin(bearing)
-        return Point(x, y)
+        x = pt[0] + dist * math.cos(bearing)
+        y = pt[1] + dist * math.sin(bearing)
+        return (x, y)
 
 
 class WeightedCharacter:
     """
     Calculates the weighted character
 
-    Character weighted by the area of the objects within `k` topological steps defined in spatial_weights.
+    Character weighted by the area of the objects within ``k`` topological steps defined in ``spatial_weights``.
 
     .. math::
         \\frac{\\sum_{i=1}^{n} {character_{i} * area_{i}}}{\\sum_{i=1}^{n} area_{i}}
 
+    Adapted from :cite:`dibble2017`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects to analyse
     values : str, list, np.array, pd.Series
-        the name of the gdf dataframe column, np.array, or pd.Series where is stored character to be weighted
+        the name of the gdf dataframe column, ``np.array``, or ``pd.Series`` where is stored character to be weighted
     spatial_weights : libpysal.weights
         spatial weights matrix - If None, Queen contiguity matrix of set order will be calculated
         based on left.
     unique_id : str
-        name of the column with unique id used as spatial_weights index.
+        name of the column with unique id used as ``spatial_weights`` index.
     areas : str, list, np.array, pd.Series (default None)
-        the name of the left dataframe column, np.array, or pd.Series where is stored area value
+        the name of the left dataframe column, ``np.array``, or ``pd.Series`` where is stored area value
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
 
     Attributes
@@ -762,11 +748,6 @@ class WeightedCharacter:
     id : Series
         Series containing used unique ID
 
-    References
-    ----------
-    Dibble J, Prelorendjos A, Romice O, et al. (2017) On the origin of spaces: Morphometric foundations of urban form evolution.
-    Environment and Planning B: Urban Analytics and City Science 46(4): 707–730.
-
     Examples
     --------
     >>> sw = libpysal.weights.DistanceBand.from_dataframe(tessellation_df, threshold=100, silence_warnings=True)
@@ -775,7 +756,9 @@ class WeightedCharacter:
     100%|██████████| 144/144 [00:00<00:00, 361.60it/s]
     """
 
-    def __init__(self, gdf, values, spatial_weights, unique_id, areas=None):
+    def __init__(
+        self, gdf, values, spatial_weights, unique_id, areas=None, verbose=True
+    ):
         self.gdf = gdf
         self.sw = spatial_weights
         self.id = gdf[unique_id]
@@ -797,7 +780,7 @@ class WeightedCharacter:
         data = data.set_index(unique_id)[[values, areas]]
 
         results_list = []
-        for index in tqdm(data.index, total=data.shape[0]):
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -819,7 +802,7 @@ class CoveredArea:
     """
     Calculates the area covered by neighbours
 
-    Total area covered by neighbours defined in spatial_weights and element itself.
+    Total area covered by neighbours defined in ``spatial_weights`` and element itself.
 
     .. math::
 
@@ -831,7 +814,9 @@ class CoveredArea:
     spatial_weights : libpysal.weights
         spatial weights matrix
     unique_id : str
-        name of the column with unique id used as spatial_weights index.
+        name of the column with unique id used as ``spatial_weights`` index.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -852,7 +837,7 @@ class CoveredArea:
 
     """
 
-    def __init__(self, gdf, spatial_weights, unique_id):
+    def __init__(self, gdf, spatial_weights, unique_id, verbose=True):
         self.gdf = gdf
         self.sw = spatial_weights
         self.id = gdf[unique_id]
@@ -861,7 +846,7 @@ class CoveredArea:
         area = data.set_index(unique_id).geometry.area
 
         results_list = []
-        for index in tqdm(area.index, total=area.shape[0]):
+        for index in tqdm(area.index, total=area.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -888,6 +873,8 @@ class PerimeterWall:
     spatial_weights : libpysal.weights, optional
         spatial weights matrix - If None, Queen contiguity matrix will be calculated
         based on gdf. It is to denote adjacent buildings (note: based on index, not ID).
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -910,15 +897,16 @@ class PerimeterWall:
     It might take a while to compute this character.
     """
 
-    def __init__(self, gdf, spatial_weights=None):
+    def __init__(self, gdf, spatial_weights=None, verbose=True):
         self.gdf = gdf
 
         if spatial_weights is None:
-            print("Calculating spatial weights...")
+
+            print("Calculating spatial weights...") if verbose else None
             from libpysal.weights import Queen
 
             spatial_weights = Queen.from_dataframe(gdf, silence_warnings=True)
-            print("Spatial weights ready...")
+            print("Spatial weights ready...") if verbose else None
         self.sw = spatial_weights
 
         # dict to store walls for each uID
@@ -926,7 +914,7 @@ class PerimeterWall:
         components = pd.Series(spatial_weights.component_labels, index=range(len(gdf)))
         geom = gdf.geometry
 
-        for i in tqdm(range(gdf.shape[0]), total=gdf.shape[0]):
+        for i in tqdm(range(gdf.shape[0]), total=gdf.shape[0], disable=not verbose):
             # if the id is already present in walls, continue (avoid repetition)
             if i in walls:
                 continue
@@ -941,7 +929,7 @@ class PerimeterWall:
                     walls[b] = dissolved.exterior.length
 
         results_list = []
-        for i in tqdm(range(gdf.shape[0]), total=gdf.shape[0]):
+        for i in tqdm(range(gdf.shape[0]), total=gdf.shape[0], disable=not verbose):
             results_list.append(walls[i])
         self.series = pd.Series(results_list, index=gdf.index)
 
@@ -951,8 +939,8 @@ class SegmentsLength:
     Calculate the cummulative and/or mean length of segments.
 
     Length of segments within set topological distance from each of them.
-    Reached topological distance should be captured by spatial_weights. If mean=False it
-    will compute sum of length, if mean=True it will compute sum and mean.
+    Reached topological distance should be captured by ``spatial_weights``. If ``mean=False`` it
+    will compute sum of length, if ``mean=True`` it will compute sum and mean.
 
     Parameters
     ----------
@@ -960,10 +948,12 @@ class SegmentsLength:
         GeoDataFrame containing streets (edges) to analyse
     spatial_weights : libpysal.weights, optional
         spatial weights matrix - If None, Queen contiguity matrix will be calculated
-        based on streets (note: spatial_weights shoud be based on index, not unique ID).
+        based on streets (note: spatial_weights should be based on index, not unique ID).
     mean : boolean, optional
         If mean=False it will compute sum of length, if mean=True it will compute
         sum and mean
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -985,22 +975,22 @@ class SegmentsLength:
     Spatial weights ready...
     """
 
-    def __init__(self, gdf, spatial_weights=None, mean=False):
+    def __init__(self, gdf, spatial_weights=None, mean=False, verbose=True):
         self.gdf = gdf
 
         if spatial_weights is None:
-            print("Calculating spatial weights...")
+            print("Calculating spatial weights...") if verbose else None
             from libpysal.weights import Queen
 
             spatial_weights = Queen.from_dataframe(gdf, silence_warnings=True)
-            print("Spatial weights ready...")
+            print("Spatial weights ready...") if verbose else None
         self.sw = spatial_weights
 
         lenghts = gdf.geometry.length
 
         sums = []
         means = []
-        for index in tqdm(gdf.index, total=gdf.shape[0]):
+        for index in tqdm(gdf.index, total=gdf.shape[0], disable=not verbose):
             neighbours = spatial_weights.neighbors[index].copy()
             if neighbours:
                 neighbours.append(index)
