@@ -2,17 +2,21 @@
 # -*- coding: utf-8 -*-
 
 # distribution.py
-# definitons of spatial distribution characters
+# definitions of spatial distribution characters
 
 import math
 import statistics
+import warnings
+from distutils.version import LooseVersion
 
 import numpy as np
 import pandas as pd
+import geopandas as gpd
 from tqdm import tqdm  # progress bar
 
 from .utils import _azimuth
 
+GPD_08 = str(gpd.__version__) >= LooseVersion("0.8.0")
 
 __all__ = [
     "Orientation",
@@ -37,10 +41,14 @@ class Orientation:
     Orientation of LineStrings is represented by the orientation of line
     connecting first and last point of the segment.
 
+    Adapted from :cite:`schirmer2015`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects to analyse
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -49,11 +57,6 @@ class Orientation:
     gdf : GeoDataFrame
         original GeoDataFrame
 
-    References
-    ----------
-    Schirmer PM and Axhausen KW (2015) A multiscale classiﬁcation of urban morphology.
-    Journal of Transport and Land Use 9(1): 101–130. (adapted)
-
     Examples
     --------
     >>> buildings_df['orientation'] = momepy.Orientation(buildings_df).series
@@ -62,7 +65,7 @@ class Orientation:
     41.05146788287027
     """
 
-    def __init__(self, gdf):
+    def __init__(self, gdf, verbose=True):
         self.gdf = gdf
         # define empty list for results
         results_list = []
@@ -70,7 +73,7 @@ class Orientation:
         def _dist(a, b):
             return math.hypot(b[0] - a[0], b[1] - a[1])
 
-        for geom in tqdm(gdf.geometry, total=gdf.shape[0]):
+        for geom in tqdm(gdf.geometry, total=gdf.shape[0], disable=not verbose):
             if geom.type in ["Polygon", "MultiPolygon", "LinearRing"]:
                 # TODO: vectorize once minimum_rotated_rectangle is in geopandas from pygeos
                 bbox = list(geom.minimum_rotated_rectangle.exterior.coords)
@@ -117,14 +120,15 @@ class SharedWallsRatio:
     Note that data needs to be topologically correct. Overlapping polygons will lead to
     incorrect results.
 
+    Adapted from :cite:`hamaina2012a`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing gdf to analyse
-    unique_id : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series with unique id
+    unique_id : (deprecated)
     perimeters : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored perimeter value
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored perimeter value
 
     Attributes
     ----------
@@ -132,70 +136,46 @@ class SharedWallsRatio:
         Series containing resulting values
     gdf : GeoDataFrame
         original GeoDataFrame
-    id : Series
-        Series containing used unique ID
     perimeters : GeoDataFrame
         Series containing used perimeters values
-    sindex : rtree spatial index
-        spatial index of gdf
-
-    References
-    ----------
-    Hamaina R, Leduc T and Moreau G (2012) Towards Urban Fabrics Characterization
-    Based on Buildings Footprints. In: Lecture Notes in Geoinformation and Cartography,
-    Berlin, Heidelberg: Springer Berlin Heidelberg, pp. 327–346. Available from:
-    https://link.springer.com/chapter/10.1007/978-3-642-29063-3_18.
 
     Examples
     --------
-    >>> buildings_df['swr'] = momepy.SharedWallsRatio(buildings_df, 'uID').series
-    100%|██████████| 144/144 [00:00<00:00, 648.72it/s]
+    >>> buildings_df['swr'] = momepy.SharedWallsRatio(buildings_df).series
     >>> buildings_df['swr'][10]
     0.3424804411228673
     """
 
-    def __init__(self, gdf, unique_id, perimeters=None):
+    def __init__(self, gdf, unique_id=None, perimeters=None):
+        if not GPD_08:
+            raise ImportError(
+                "The 'geopandas' >= 0.8.0 package is required to use SharedWallsRatio."
+            )
+        if unique_id is not None:
+            warnings.warn(
+                "unique_id is deprecated and will be removed in v0.4.", FutureWarning,
+            )
+
         self.gdf = gdf
 
-        gdf = gdf.copy()
-        self.sindex = gdf.sindex  # define rtree index
-        # define empty list for results
-        results_list = []
-
         if perimeters is None:
-            gdf["mm_p"] = gdf.geometry.length
-            perimeters = "mm_p"
+            self.perimeters = gdf.geometry.length
+        elif isinstance(perimeters, str):
+            self.perimeters = gdf[perimeters]
         else:
-            if not isinstance(perimeters, str):
-                gdf["mm_p"] = perimeters
-                perimeters = "mm_p"
+            self.perimeters = perimeters
 
-        self.perimeters = gdf[perimeters]
+        inp, res = gdf.sindex.query_bulk(gdf.geometry, predicate="intersects")
+        left = gdf.geometry.take(inp).reset_index(drop=True)
+        right = gdf.geometry.take(res).reset_index(drop=True)
+        intersections = left.intersection(right).length
+        results = (
+            intersections.groupby(inp).sum().reset_index(drop=True)
+            - self.perimeters.reset_index(drop=True)
+        ) / self.perimeters.reset_index(drop=True)
+        results.index = gdf.index
 
-        if not isinstance(unique_id, str):
-            gdf["mm_uid"] = unique_id
-            unique_id = "mm_uid"
-        self.id = gdf[unique_id]
-
-        gdf["_bounds"] = gdf.geometry.bounds.apply(list, axis=1)
-        for i, row in tqdm(
-            enumerate(
-                gdf[[perimeters, "_bounds", gdf._geometry_column_name]].itertuples()
-            ),
-            total=gdf.shape[0],
-        ):
-            neighbors = list(self.sindex.intersection(row[2]))
-            neighbors.remove(i)
-
-            # if no neighbour exists
-            length = 0
-            if not neighbors:
-                results_list.append(0)
-            else:
-                length = gdf.iloc[neighbors].intersection(row[3]).length.sum()
-                results_list.append(length / row[1])
-
-        self.series = pd.Series(results_list, index=gdf.index)
+        self.series = results
 
 
 class StreetAlignment:
@@ -204,8 +184,8 @@ class StreetAlignment:
 
     Orientation of street segment is represented by the orientation of line
     connecting first and last point of the segment. Network ID linking each object
-    to specific street segment is needed. Can be generated by :py:func:`momepy.get_network_id`.
-    Either network_id or left_network_id and right_network_id are required.
+    to specific street segment is needed. Can be generated by :func:`momepy.get_network_id`.
+    Either ``network_id`` or both ``left_network_id`` and ``right_network_id`` are required.
 
     .. math::
         \\left|{\\textit{building orientation} - \\textit{street orientation}}\\right|
@@ -217,15 +197,15 @@ class StreetAlignment:
     right : GeoDataFrame
         GeoDataFrame containing street network
     orientations : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored object orientation value
-        (can be calculated using :py:func:`momepy.Orientation`)
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored object orientation value
+        (can be calculated using :class:`momepy.Orientation`)
     network_id : str (default None)
         the name of the column storing network ID in both left and right
     left_network_id : str, list, np.array, pd.Series (default None)
-        the name of the left dataframe column, np.array, or pd.Series where is stored object network ID
+        the name of the left dataframe column, ``np.array``, or ``pd.Series`` where is stored object network ID
     right_network_id : str, list, np.array, pd.Series (default None)
-        the name of the right dataframe column, np.array, or pd.Series of streets with unique network id (has to be defined beforehand)
-        (can be defined using :py:func:`momepy.elements.unique_id`)
+        the name of the right dataframe column, ``np.array``, or ``pd.Series`` of streets with unique network id (has to be defined beforehand)
+        (can be defined using :func:`momepy.unique_id`)
 
     Attributes
     ----------
@@ -245,7 +225,6 @@ class StreetAlignment:
     Examples
     --------
     >>> buildings_df['street_alignment'] = momepy.StreetAlignment(buildings_df, streets_df, 'orientation', 'nID', 'nID').series
-    100%|██████████| 144/144 [00:00<00:00, 529.94it/s]
     >>> buildings_df['street_alignment'][0]
     0.29073888476702336
     """
@@ -293,7 +272,7 @@ class StreetAlignment:
             right_network_id = "mm_nis"
         self.right_network_id = right[right_network_id]
 
-        right["_orientation"] = Orientation(right).series
+        right["_orientation"] = Orientation(right, verbose=False).series
 
         merged = left[[left_network_id, orientations]].merge(
             right[[right_network_id, "_orientation"]],
@@ -319,15 +298,15 @@ class CellAlignment:
     right : GeoDataFrame
         GeoDataFrame containing tessellation cells (or relevant spatial units)
     left_orientations : str, list, np.array, pd.Series
-        the name of the left dataframe column, np.array, or pd.Series where is stored object orientation value
-        (can be calculated using :py:func:`momepy.orientation`)
+        the name of the left dataframe column, ``np.array``, or ``pd.Series`` where is stored object orientation value
+        (can be calculated using :class:`momepy.Orientation`)
     right_orientations : str, list, np.array, pd.Series
-        the name of the right dataframe column, np.array, or pd.Series where is stored object orientation value
-        (can be calculated using :py:func:`momepy.orientation`)
+        the name of the right dataframe column, ``np.array``, or ``pd.Series`` where is stored object orientation value
+        (can be calculated using :class:`momepy.Orientation`)
     left_unique_id : str
-        the name of the left dataframe column with unique id shared between left and right gdf
+        the name of the ``left`` dataframe column with unique id shared between ``left`` and ``right`` gdf
     right_unique_id : str
-        the name of the right dataframe column with unique id shared between left and right gdf
+        the name of the ``right`` dataframe column with unique id shared between ``left`` and ``right`` gdf
 
     Attributes
     ----------
@@ -349,7 +328,6 @@ class CellAlignment:
     Examples
     --------
     >>> buildings_df['cell_alignment'] = momepy.CellAlignment(buildings_df, tessellation_df, 'bl_orient', 'tes_orient', 'uID', 'uID').series
-    100%|██████████| 144/144 [00:00<00:00, 799.09it/s]
     >>> buildings_df['cell_alignment'][0]
     0.8795123936951939
 
@@ -408,10 +386,12 @@ class Alignment:
     spatial_weights : libpysal.weights, optional
         spatial weights matrix
     orientations : str, list, np.array, pd.Series
-        the name of the left dataframe column, np.array, or pd.Series where is stored object orientation value
-        (can be calculated using :py:func:`momepy.orientation`)
+        the name of the left dataframe column, ``np.array``, or ``pd.Series`` where is stored object orientation value
+        (can be calculated using :class:`momepy.Orientation`)
     unique_id : str
-        name of the column with unique id used as spatial_weights index.
+        name of the column with unique id used as ``spatial_weights`` index.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -434,7 +414,7 @@ class Alignment:
     18.299481296455237
     """
 
-    def __init__(self, gdf, spatial_weights, unique_id, orientations):
+    def __init__(self, gdf, spatial_weights, unique_id, orientations, verbose=True):
         self.gdf = gdf
         self.sw = spatial_weights
         self.id = gdf[unique_id]
@@ -450,7 +430,9 @@ class Alignment:
         data = gdf.set_index(unique_id)[orientations]
 
         # iterating over rows one by one
-        for index, orient in tqdm(data.iteritems(), total=data.shape[0]):
+        for index, orient in tqdm(
+            data.iteritems(), total=data.shape[0], disable=not verbose
+        ):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -468,13 +450,15 @@ class Alignment:
 
 class NeighborDistance:
     """
-    Calculate the mean distance to adjacent buildings (based on spatial_weights)
+    Calculate the mean distance to adjacent buildings (based on ``spatial_weights``)
 
-    If no neighbours are found, return np.nan.
+    If no neighbours are found, return ``np.nan``.
 
     .. math::
         \\frac{1}{n}\\sum_{i=1}^n dist_i=\\frac{dist_1+dist_2+\\cdots+dist_n}{n}
 
+    Adapted from :cite:`schirmer2015`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
@@ -482,7 +466,9 @@ class NeighborDistance:
     spatial_weights : libpysal.weights
         spatial weights matrix based on unique_id
     unique_id : str
-        name of the column with unique id used as spatial_weights index.
+        name of the column with unique id used as ``spatial_weights`` index.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -495,11 +481,6 @@ class NeighborDistance:
     id : Series
         Series containing used unique ID
 
-    References
-    ----------
-    Schirmer PM and Axhausen KW (2015) A multiscale classiﬁcation of urban morphology.
-    Journal of Transport and Land Use 9(1): 101–130. (adapted)
-
     Examples
     --------
     >>> buildings_df['neighbour_distance'] = momepy.NeighborDistance(buildings_df, sw, 'uID').series
@@ -508,7 +489,7 @@ class NeighborDistance:
     29.18589019096464
     """
 
-    def __init__(self, gdf, spatial_weights, unique_id):
+    def __init__(self, gdf, spatial_weights, unique_id, verbose=True):
         self.gdf = gdf
         self.sw = spatial_weights
         self.id = gdf[unique_id]
@@ -518,7 +499,9 @@ class NeighborDistance:
         data = gdf.set_index(unique_id).geometry
 
         # iterating over rows one by one
-        for index, geom in tqdm(data.iteritems(), total=data.shape[0]):
+        for index, geom in tqdm(
+            data.iteritems(), total=data.shape[0], disable=not verbose
+        ):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index]
                 building_neighbours = data.loc[neighbours]
@@ -539,7 +522,7 @@ class MeanInterbuildingDistance:
     Calculate the mean interbuilding distance
 
     Interbuilding distances are calculated between buildings on adjacent cells based on
-    `spatial_weights`, while the extent is defined in `spatial_weights_higher`.
+    ``spatial_weights``, while the extent is defined in ``spatial_weights_higher``.
 
     .. math::
 
@@ -549,14 +532,16 @@ class MeanInterbuildingDistance:
     gdf : GeoDataFrame
         GeoDataFrame containing objects to analyse
     unique_id : str
-        name of the column with unique id used as spatial_weights index
+        name of the column with unique id used as ``spatial_weights`` index
     spatial_weights : libpysal.weights
         spatial weights matrix
     spatial_weights_higher : libpysal.weights, optional
-        spatial weights matrix - If None, Queen contiguity of higher order will be calculated
-        based on spatial_weights
+        spatial weights matrix - If None, Queen contiguity of a higher ``order`` will be calculated
+        based on ``spatial_weights``
     order : int
         Order of Queen contiguity
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -581,9 +566,6 @@ class MeanInterbuildingDistance:
     --------
     >>> buildings_df['mean_interbuilding_distance'] = momepy.MeanInterbuildingDistance(buildings_df, sw, 'uID').series
     Generating weights matrix (Queen) of 3 topological steps...
-    Generating adjacency matrix based on weights matrix...
-    Computing interbuilding distances...
-    100%|██████████| 746/746 [00:03<00:00, 200.14it/s]
     Computing mean interbuilding distances...
     100%|██████████| 144/144 [00:00<00:00, 317.42it/s]
     >>> buildings_df['mean_interbuilding_distance'][0]
@@ -591,7 +573,13 @@ class MeanInterbuildingDistance:
     """
 
     def __init__(
-        self, gdf, spatial_weights, unique_id, spatial_weights_higher=None, order=3
+        self,
+        gdf,
+        spatial_weights,
+        unique_id,
+        spatial_weights_higher=None,
+        order=3,
+        verbose=True,
     ):
         self.gdf = gdf
         self.sw = spatial_weights
@@ -599,10 +587,8 @@ class MeanInterbuildingDistance:
 
         if spatial_weights_higher is None:
             print(
-                "Generating weights matrix (Queen) of {} topological steps...".format(
-                    order
-                )
-            )
+                f"Generating weights matrix (Queen) of {order} topological steps..."
+            ) if verbose else None
             self.order = order
             from momepy import sw_high
 
@@ -610,35 +596,22 @@ class MeanInterbuildingDistance:
             spatial_weights_higher = sw_high(k=order, weights=spatial_weights)
         self.sw_higher = spatial_weights_higher
 
-        data = gdf.set_index(unique_id).copy()
+        data = gdf.set_index(unique_id).geometry
 
         # define empty list for results
         results_list = []
 
-        print("Generating adjacency matrix based on weights matrix...")
         # define adjacency list from lipysal
         adj_list = spatial_weights.to_adjlist()
-        adj_list["distance"] = -1
-
-        print("Computing interbuilding distances...")
-        # measure each interbuilding distance of neighbours and save them to adjacency list
-        for row in tqdm(adj_list.itertuples(), total=adj_list.shape[0]):
-            inverted = adj_list[(adj_list.focal == row.neighbor)][
-                (adj_list.neighbor == row.focal)
-            ].iloc[0]["distance"]
-            if inverted == -1:
-                building_object = data.loc[row.focal]
-
-                building_neighbour = data.loc[row.neighbor]
-                adj_list.loc[row.Index, "distance"] = building_neighbour.iloc[
-                    0
-                ].distance(building_object.iloc[0])
-            else:
-                adj_list.at[row.Index, "distance"] = inverted
+        adj_list["distance"] = (
+            data.loc[adj_list.focal]
+            .reset_index()
+            .distance(data.loc[adj_list.neighbor].reset_index())
+        )
 
         print("Computing mean interbuilding distances...")
         # iterate over objects to get the final values
-        for uid in tqdm(data.index, total=data.shape[0]):
+        for uid in tqdm(data.index, total=data.shape[0], disable=not verbose):
             # define neighbours based on weights matrix defining analysis area
             if uid in spatial_weights_higher.neighbors.keys():
                 neighbours = spatial_weights_higher.neighbors[uid].copy()
@@ -677,72 +650,59 @@ class NeighboringStreetOrientationDeviation:
         original GeoDataFrame
     orientation : Series
         Series containing used street orientation values
-    sindex : rtree spatial index
-        spatial index of gdf
 
     Examples
     --------
     >>> streets_df['orient_dev'] = momepy.NeighboringStreetOrientationDeviation(streets_df).series
-    Preparing street orientations...
-    Generating spatial index...
-    100%|██████████| 33/33 [00:00<00:00, 249.02it/s]
     >>> streets_df['orient_dev'][6]
     7.043096518688273
     """
 
     def __init__(self, gdf):
+        if not GPD_08:
+            raise ImportError(
+                "The 'geopandas' >= 0.8.0 package is required to use "
+                "NeighboringStreetOrientationDeviation."
+            )
         self.gdf = gdf
+        self.orientation = gdf.geometry.apply(self._orient)
 
-        results_list = []
-        gdf = gdf.copy()
+        inp, res = gdf.sindex.query_bulk(gdf.geometry, predicate="intersects")
+        itself = inp == res
+        inp = inp[~itself]
+        res = res[~itself]
 
-        # iterating over rows one by one
-        print(" Preparing street orientations...")
-        for geom in tqdm(gdf.geometry, total=gdf.shape[0]):
+        left = self.orientation.take(inp).reset_index(drop=True)
+        right = self.orientation.take(res).reset_index(drop=True)
+        deviations = (left - right).abs()
 
-            start = geom.coords[0]
-            end = geom.coords[-1]
-            az = _azimuth(start, end)
-            if 90 > az >= 45:
-                diff = az - 45
-                az = az - 2 * diff
-            elif 135 > az >= 90:
-                diff = az - 90
-                az = az - 2 * diff
-                diff = az - 45
-                az = az - 2 * diff
-            elif 181 > az >= 135:
-                diff = az - 135
-                az = az - 2 * diff
-                diff = az - 90
-                az = az - 2 * diff
-                diff = az - 45
-                az = az - 2 * diff
-            results_list.append(az)
-        self.orientation = pd.Series(results_list, index=gdf.index)
+        results = deviations.groupby(inp).mean()
 
-        gdf["tmporient"] = self.orientation
+        match = gdf.iloc[list(results.index)]
+        match["result"] = results.to_list()
 
-        print(" Generating spatial index...")
-        self.sindex = gdf.sindex
-        results_list = []
+        self.series = match.result
 
-        for row in tqdm(gdf.itertuples(), total=gdf.shape[0]):
-            possible_neighbors_idx = list(self.sindex.intersection(row.geometry.bounds))
-            possible_neighbours = gdf.iloc[possible_neighbors_idx]
-            neighbors = possible_neighbours[
-                possible_neighbours.intersects(row.geometry)
-            ]
-            neighbors.drop([row.Index])
-
-            deviations = np.abs(neighbors.tmporient - row.tmporient)
-
-            if not deviations.empty:
-                results_list.append(np.mean(deviations))
-            else:
-                results_list.append(0)
-
-        self.series = pd.Series(results_list, index=gdf.index)
+    def _orient(self, geom):
+        start = geom.coords[0]
+        end = geom.coords[-1]
+        az = _azimuth(start, end)
+        if 90 > az >= 45:
+            diff = az - 45
+            az = az - 2 * diff
+        elif 135 > az >= 90:
+            diff = az - 90
+            az = az - 2 * diff
+            diff = az - 45
+            az = az - 2 * diff
+        elif 181 > az >= 135:
+            diff = az - 135
+            az = az - 2 * diff
+            diff = az - 90
+            az = az - 2 * diff
+            diff = az - 45
+            az = az - 2 * diff
+        return az
 
 
 class BuildingAdjacency:
@@ -751,10 +711,9 @@ class BuildingAdjacency:
 
     Building adjacency reflects how much buildings tend to join together into larger structures.
     It is calculated as a ratio of joined built-up structures and buildings within
-    the extent defined in `spatial_weights_higher`.
-
-    .. math::
+    the extent defined in ``spatial_weights_higher``.
 
+    Adapted from :cite:`vanderhaegen2017`.
 
     Parameters
     ----------
@@ -763,10 +722,12 @@ class BuildingAdjacency:
     spatial_weights_higher : libpysal.weights
         spatial weights matrix
     unique_id : str
-        name of the column with unique id used as spatial_weights index
+        name of the column with unique id used as ``spatial_weights`` index
     spatial_weights : libpysal.weights, optional
         spatial weights matrix - If None, Queen contiguity matrix will be calculated
         based on gdf. It is to denote adjacent buildings (note: based on unique ID).
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -781,24 +742,19 @@ class BuildingAdjacency:
     sw : libpysal.weights
         spatial weights matrix
 
-    References
-    ----------
-    Vanderhaegen S and Canters F (2017) Mapping urban form and function at city
-    block level using spatial metrics. Landscape and Urban Planning 167: 399–409.
-
     Examples
     --------
     >>> buildings_df['adjacency'] = momepy.BuildingAdjacency(buildings_df, swh, unique_id='uID').series
     Calculating spatial weights...
     Spatial weights ready...
-    100%|██████████| 144/144 [00:00<00:00, 9301.73it/s]
-    Calculating adjacency...
-    100%|██████████| 144/144 [00:00<00:00, 335.55it/s]
+    Calculating adjacency: 100%|██████████| 144/144 [00:00<00:00, 335.55it/s]
     >>> buildings_df['adjacency'][10]
     0.23809523809523808
     """
 
-    def __init__(self, gdf, spatial_weights_higher, unique_id, spatial_weights=None):
+    def __init__(
+        self, gdf, spatial_weights_higher, unique_id, spatial_weights=None, verbose=True
+    ):
         self.gdf = gdf
         self.sw_higher = spatial_weights_higher
         self.id = gdf[unique_id]
@@ -806,19 +762,23 @@ class BuildingAdjacency:
 
         # if weights matrix is not passed, generate it from gdf
         if spatial_weights is None:
-            print("Calculating spatial weights...")
+            print("Calculating spatial weights...") if verbose else None
             from libpysal.weights import Queen
 
             spatial_weights = Queen.from_dataframe(
                 gdf, silence_warnings=True, ids=unique_id
             )
-            print("Spatial weights ready...")
+            print("Spatial weights ready...") if verbose else None
 
         self.sw = spatial_weights
         patches = dict(zip(gdf[unique_id], spatial_weights.component_labels))
 
-        print("Calculating adjacency...")
-        for uid in tqdm(self.id, total=gdf.shape[0]):
+        for uid in tqdm(
+            self.id,
+            total=gdf.shape[0],
+            disable=not verbose,
+            desc="Calculating adjacency",
+        ):
             if uid in spatial_weights_higher.neighbors.keys():
                 neighbours = spatial_weights_higher.neighbors[uid].copy()
                 if neighbours:
@@ -838,12 +798,12 @@ class BuildingAdjacency:
 
 class Neighbors:
     """
-    Calculate the number of neighbours captured by spatial_weights
+    Calculate the number of neighbours captured by ``spatial_weights``
 
-    If weighted=True, number of neighbours will be divided by the perimeter of object
+    If ``weighted=True``, number of neighbours will be divided by the perimeter of object
     to return relative value.
 
-    .. math::
+    Adapted from :cite:`hermosilla2012`.
 
 
     Parameters
@@ -853,9 +813,11 @@ class Neighbors:
     spatial_weights : libpysal.weights
         spatial weights matrix
     unique_id : str
-        name of the column with unique id used as spatial_weights index
+        name of the column with unique id used as ``spatial_weights`` index
     weighted : bool (default False)
-        if weighted=True, number of neighbours will be divided by the perimeter of object, to return relative value
+        if ``True``, number of neighbours will be divided by the perimeter of object, to return relative value
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -872,12 +834,6 @@ class Neighbors:
     weighted : bool
         used weighted value
 
-    References
-    ----------
-    Hermosilla T, Ruiz LA, Recio JA, et al. (2012) Assessing contextual descriptive features
-    for plot-based classification of urban areas. Landscape and Urban Planning, Elsevier B.V.
-    106(1): 124–137.
-
     Examples
     --------
     >>> sw = libpysal.weights.contiguity.Queen.from_dataframe(tessellation_df, ids='uID')
@@ -887,7 +843,7 @@ class Neighbors:
     4
     """
 
-    def __init__(self, gdf, spatial_weights, unique_id, weighted=False):
+    def __init__(self, gdf, spatial_weights, unique_id, weighted=False, verbose=True):
         self.gdf = gdf
         self.sw = spatial_weights
         self.id = gdf[unique_id]
@@ -895,7 +851,9 @@ class Neighbors:
 
         neighbours = []
         for index, geom in tqdm(
-            gdf.set_index(unique_id).geometry.iteritems(), total=gdf.shape[0]
+            gdf.set_index(unique_id).geometry.iteritems(),
+            total=gdf.shape[0],
+            disable=not verbose,
         ):
             if index in spatial_weights.neighbors.keys():
                 if weighted is True:
