@@ -2,40 +2,50 @@
 # -*- coding: utf-8 -*-
 
 # diversity.py
-# definitons of diversity characters
+# definitions of diversity characters
 
 import numpy as np
 import pandas as pd
 import scipy as sp
 from tqdm import tqdm  # progress bar
 
-__all__ = ["Range", "Theil", "Simpson", "Gini", "Shannon", "Unique"]
+__all__ = [
+    "Range",
+    "Theil",
+    "Simpson",
+    "Gini",
+    "Shannon",
+    "Unique",
+    "simpson_diversity",
+    "shannon_diversity",
+]
 
 
 class Range:
     """
-    Calculates the range of values within neighbours defined in `spatial_weights`.
+    Calculates the range of values within neighbours defined in ``spatial_weights``.
 
-    Uses `scipy.stats.iqr` under the hood.
-
-    .. math::
+    Uses ``scipy.stats.iqr`` under the hood.
 
+    Adapted from :cite:`dibble2017`.
 
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing morphological tessellation
     values : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored character value.
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored character value.
     spatial_weights : libpysal.weights
         spatial weights matrix
     unique_id : str
-        name of the column with unique id used as spatial_weights index
+        name of the column with unique id used as ``spatial_weights`` index
     rng : Two-element sequence containing floats in range of [0,100], optional
         Percentiles over which to compute the range. Each must be
         between 0 and 100, inclusive. The order of the elements is not important.
     **kwargs : keyword arguments
-        optional arguments for `scipy.stats.iqr`
+        optional arguments for ``scipy.stats.iqr``
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -54,11 +64,6 @@ class Range:
     kwargs : dict
         kwargs
 
-    References
-    ----------
-    Dibble J, Prelorendjos A, Romice O, et al. (2017) On the origin of spaces: Morphometric foundations of urban form evolution.
-    Environment and Planning B: Urban Analytics and City Science 46(4): 707–730.
-
     Examples
     --------
     >>> sw = momepy.sw_high(k=3, gdf=tessellation_df, ids='uID')
@@ -68,7 +73,16 @@ class Range:
 
     """
 
-    def __init__(self, gdf, values, spatial_weights, unique_id, rng=(0, 100), **kwargs):
+    def __init__(
+        self,
+        gdf,
+        values,
+        spatial_weights,
+        unique_id,
+        rng=(0, 100),
+        verbose=True,
+        **kwargs
+    ):
         self.gdf = gdf
         self.sw = spatial_weights
         self.id = gdf[unique_id]
@@ -85,7 +99,7 @@ class Range:
         data = data.set_index(unique_id)[values]
 
         results_list = []
-        for index in tqdm(data.index, total=data.shape[0]):
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -103,9 +117,9 @@ class Range:
 
 class Theil:
     """
-    Calculates the Theil measure of inequality of values within neighbours defined in `spatial_weights`.
+    Calculates the Theil measure of inequality of values within neighbours defined in ``spatial_weights``.
 
-    Uses `inequality.theil.Theil` under the hood. Requires 'inequality' package.
+    Uses ``inequality.theil.Theil`` under the hood. Requires '`inequality`' package.
 
     .. math::
 
@@ -116,14 +130,16 @@ class Theil:
     gdf : GeoDataFrame
         GeoDataFrame containing morphological tessellation
     values : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored character value.
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored character value.
     spatial_weights : libpysal.weights
         spatial weights matrix
     unique_id : str
-        name of the column with unique id used as spatial_weights index
+        name of the column with unique id used as ``spatial_weights`` index
     rng : Two-element sequence containing floats in range of [0,100], optional
         Percentiles over which to compute the range. Each must be
         between 0 and 100, inclusive. The order of the elements is not important.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -147,7 +163,7 @@ class Theil:
     100%|██████████| 144/144 [00:00<00:00, 597.37it/s]
     """
 
-    def __init__(self, gdf, values, spatial_weights, unique_id, rng=None):
+    def __init__(self, gdf, values, spatial_weights, unique_id, rng=None, verbose=True):
         try:
             from inequality.theil import Theil
         except ImportError:
@@ -171,7 +187,7 @@ class Theil:
             from momepy import limit_range
 
         results_list = []
-        for index in tqdm(data.index, total=data.shape[0]):
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -192,44 +208,43 @@ class Theil:
 
 class Simpson:
     """
-    Calculates the Simpson\'s diversity index of values within neighbours defined in `spatial_weights`.
+    Calculates the Simpson\'s diversity index of values within neighbours defined in ``spatial_weights``.
 
-    Uses `mapclassify.classifiers` under the hood for binning. Requires `mapclassify>=.2.1.0` dependency.
+    Uses ``mapclassify.classifiers`` under the hood for binning. Requires ``mapclassify>=.2.1.0`` dependency.
 
     .. math::
 
         \\lambda=\\sum_{i=1}^{R} p_{i}^{2}
 
+    Adapted from :cite:`feliciotti2018`.
+
     Parameters
     ----------
-    objects : GeoDataFrame
+    gdf : GeoDataFrame
         GeoDataFrame containing morphological tessellation
     values : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored character value.
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored character value.
     spatial_weights : libpysal.weights, optional
         spatial weights matrix - If None, Queen contiguity matrix of set order will be calculated
         based on objects.
-    order : int
-        order of Queen contiguity
-    binning : str
-        One of mapclassify classification schemes
-        Options are BoxPlot, EqualInterval, FisherJenks,
-        FisherJenksSampled, HeadTailBreaks, JenksCaspall,
-        JenksCaspallForced, JenksCaspallSampled, MaxPClassifier,
-        MaximumBreaks, NaturalBreaks, Quantiles, Percentiles, StdMean,
-        UserDefined
+    unique_id : str
+        name of the column with unique id used as ``spatial_weights`` index
+    binning : str (default 'HeadTailBreaks')
+        One of mapclassify classification schemes. For details see
+        `mapclassify API documentation <http://pysal.org/mapclassify/api.html>`_.
     gini_simpson : bool (default False)
-        return Gini-Simpson index instead of Simpson index (1 - λ)
+        return Gini-Simpson index instead of Simpson index (``1 - λ``)
     inverse : bool (default False)
-        return Inverse Simpson index instead of Simpson index (1 / λ)
+        return Inverse Simpson index instead of Simpson index (``1 / λ``)
     categorical : bool (default False)
-        treat values as categories (will not use binning)
+        treat values as categories (will not use ``binning``)
     categories : list-like (default None)
-        list of categories. If None values.unique() is used.
+        list of categories. If None ``values.unique()`` is used.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
     **classification_kwds : dict
         Keyword arguments for classification scheme
-        For details see mapclassify documentation:
-        https://pysal.org/mapclassify
+        For details see `mapclassify documentation <https://pysal.org/mapclassify>`_.
 
     Attributes
     ----------
@@ -250,16 +265,15 @@ class Simpson:
     classification_kwds : dict
         classification_kwds
 
-    References
-    ----------
-    Feliciotti A (2018) RESILIENCE AND URBAN DESIGN:A SYSTEMS APPROACH TO THE STUDY OF RESILIENCE
-    IN URBAN FORM. LEARNING FROM THE CASE OF GORBALS. Glasgow.
-
     Examples
     --------
     >>> sw = momepy.sw_high(k=3, gdf=tessellation_df, ids='uID')
     >>> tessellation_df['area_Simpson'] = mm.Simpson(tessellation_df, 'area', sw, 'uID').series
     100%|██████████| 144/144 [00:00<00:00, 455.83it/s]
+
+    See also
+    --------
+    momepy.simpson_diversity : Calculates the Simpson\'s diversity index of data
     """
 
     def __init__(
@@ -273,6 +287,7 @@ class Simpson:
         inverse=False,
         categorical=False,
         categories=None,
+        verbose=True,
         **classification_kwds
     ):
         if not categorical:
@@ -308,16 +323,18 @@ class Simpson:
                 values = "mm_v"
         self.values = data[values]
 
-        if not categorical:
-            self.bins = schemes[binning](data[values], **classification_kwds).bins
-
         data = data.set_index(unique_id)[values]
 
         if not categories:
             categories = data.unique()
 
+        if not categorical:
+            self.bins = schemes[binning](data, **classification_kwds).bins
+        else:
+            self.bins = categories
+
         results_list = []
-        for index in tqdm(data.index, total=data.shape[0]):
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -326,16 +343,14 @@ class Simpson:
                     neighbours = [index]
                 values_list = data.loc[neighbours]
 
-                if categorical:
-                    counts = values_list.value_counts().to_dict()
-                    for c in categories:
-                        if c not in counts.keys():
-                            counts[c] = 0
-                else:
-                    sample_bins = classifiers.UserDefined(values_list, self.bins)
-                    counts = dict(zip(self.bins, sample_bins.counts))
-
-                results_list.append(self._simpson_di(counts))
+                results_list.append(
+                    simpson_diversity(
+                        values_list,
+                        self.bins,
+                        categorical=categorical,
+                        categories=categories,
+                    )
+                )
             else:
                 results_list.append(np.nan)
 
@@ -346,32 +361,69 @@ class Simpson:
         else:
             self.series = pd.Series(results_list, index=gdf.index)
 
-    def _simpson_di(self, data):
 
-        """ Given a hash { 'species': count } , returns the Simpson Diversity Index
+def simpson_diversity(data, bins=None, categorical=False, categories=None):
+    """
+    Calculates the Simpson\'s diversity index of data. Helper function for
+    :py:class:`momepy.Simpson`.
 
-        >>> _simpson_di({'a': 10, 'b': 20, 'c': 30,})
-        0.3888888888888889
+    .. math::
 
-        https://gist.github.com/martinjc/f227b447791df8c90568
-        """
+        \\lambda=\\sum_{i=1}^{R} p_{i}^{2}
 
-        def p(n, N):
-            """ Relative abundance """
-            if n == 0:
-                return 0
-            return float(n) / N
+    Formula adapted from https://gist.github.com/martinjc/f227b447791df8c90568.
+    
+    Parameters
+    ----------
+    data : GeoDataFrame
+        GeoDataFrame containing morphological tessellation
+    bins : array, optional
+        array of top edges of classification bins. Result of binnng.bins.
+    categorical : bool (default False)
+        treat values as categories (will not use ``bins``)
+    categories : list-like (default None)
+        list of categories
 
-        N = sum(data.values())
+    Returns
+    -------
+    float
+        Simpson's diversity index
+    
+    See also
+    --------
+    momepy.Simpson : Calculates the Simpson\'s diversity index of values within neighbours
+    """
+    if not categorical:
+        try:
+            import mapclassify as mc
+        except ImportError:
+            raise ImportError("The 'mapclassify' package is required")
 
-        return sum(p(n, N) ** 2 for n in data.values() if n != 0)
+    def p(n, N):
+        """ Relative abundance """
+        if n == 0:
+            return 0
+        return float(n) / N
+
+    if categorical:
+        counts = data.value_counts().to_dict()
+        for c in categories:
+            if c not in counts.keys():
+                counts[c] = 0
+    else:
+        sample_bins = mc.UserDefined(data, bins)
+        counts = dict(zip(bins, sample_bins.counts))
+
+    N = sum(counts.values())
+
+    return sum(p(n, N) ** 2 for n in counts.values() if n != 0)
 
 
 class Gini:
     """
-    Calculates the Gini index of values within neighbours defined in `spatial_weights`.
+    Calculates the Gini index of values within neighbours defined in ``spatial_weights``.
 
-    Uses `inequality.gini.Gini` under the hood. Requires 'inequality' package.
+    Uses ``inequality.gini.Gini`` under the hood. Requires '`inequality`' package.
 
     .. math::
 
@@ -380,14 +432,16 @@ class Gini:
     gdf : GeoDataFrame
         GeoDataFrame containing morphological tessellation
     values : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored character value.
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored character value.
     spatial_weights : libpysal.weights
         spatial weights matrix
     unique_id : str
-        name of the column with unique id used as spatial_weights index
+        name of the column with unique id used as ``spatial_weights`` index
     rng : Two-element sequence containing floats in range of [0,100], optional
         Percentiles over which to compute the range. Each must be
         between 0 and 100, inclusive. The order of the elements is not important.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -411,7 +465,7 @@ class Gini:
     100%|██████████| 144/144 [00:00<00:00, 597.37it/s]
     """
 
-    def __init__(self, gdf, values, spatial_weights, unique_id, rng=None):
+    def __init__(self, gdf, values, spatial_weights, unique_id, rng=None, verbose=True):
         try:
             from inequality.gini import Gini
         except ImportError:
@@ -441,7 +495,7 @@ class Gini:
             from momepy import limit_range
 
         results_list = []
-        for index in tqdm(data.index, total=data.shape[0]):
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -462,9 +516,9 @@ class Gini:
 
 class Shannon:
     """
-    Calculates the Shannon index of values within neighbours defined in `spatial_weights`.
+    Calculates the Shannon index of values within neighbours defined in ``spatial_weights``.
 
-    Uses `mapclassify.classifiers` under the hood for binning. Requires `mapclassify>=.2.1.0` dependency.
+    Uses ``mapclassify.classifiers`` under the hood for binning. Requires ``mapclassify>=.2.1.0`` dependency.
 
     .. math::
 
@@ -472,30 +526,27 @@ class Shannon:
 
     Parameters
     ----------
-    objects : GeoDataFrame
+    gdf : GeoDataFrame
         GeoDataFrame containing morphological tessellation
     values : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored character value.
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored character value.
     spatial_weights : libpysal.weights, optional
         spatial weights matrix - If None, Queen contiguity matrix of set order will be calculated
         based on objects.
-    order : int
-        order of Queen contiguity
+    unique_id : str
+        name of the column with unique id used as ``spatial_weights`` index
     binning : str
-        One of mapclassify classification schemes
-        Options are BoxPlot, EqualInterval, FisherJenks,
-        FisherJenksSampled, HeadTailBreaks, JenksCaspall,
-        JenksCaspallForced, JenksCaspallSampled, MaxPClassifier,
-        MaximumBreaks, NaturalBreaks, Quantiles, Percentiles, StdMean,
-        UserDefined
+        One of mapclassify classification schemes. For details see
+        `mapclassify API documentation <http://pysal.org/mapclassify/api.html>`_.
     categorical : bool (default False)
         treat values as categories (will not use binning)
     categories : list-like (default None)
         list of categories. If None values.unique() is used.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
     **classification_kwds : dict
         Keyword arguments for classification scheme
-        For details see mapclassify documentation:
-        https://pysal.org/mapclassify
+        For details see `mapclassify documentation <https://pysal.org/mapclassify>`_.
 
     Attributes
     ----------
@@ -516,10 +567,6 @@ class Shannon:
     classification_kwds : dict
         classification_kwds
 
-    References
-    ----------
-
-
     Examples
     --------
     >>> sw = momepy.sw_high(k=3, gdf=tessellation_df, ids='uID')
@@ -536,6 +583,7 @@ class Shannon:
         binning="HeadTailBreaks",
         categorical=False,
         categories=None,
+        verbose=True,
         **classification_kwds
     ):
         if not categorical:
@@ -569,16 +617,18 @@ class Shannon:
                 values = "mm_v"
         self.values = data[values]
 
-        if not categorical:
-            self.bins = schemes[binning](data[values], **classification_kwds).bins
-
         data = data.set_index(unique_id)[values]
 
         if not categories:
             categories = data.unique()
 
+        if not categorical:
+            self.bins = schemes[binning](data, **classification_kwds).bins
+        else:
+            self.bins = categories
+
         results_list = []
-        for index in tqdm(data.index, total=data.shape[0]):
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
@@ -587,46 +637,84 @@ class Shannon:
                     neighbours = [index]
                 values_list = data.loc[neighbours]
 
-                if categorical:
-                    counts = values_list.value_counts().to_dict()
-                    for c in categories:
-                        if c not in counts.keys():
-                            counts[c] = 0
-                else:
-                    sample_bins = classifiers.UserDefined(values_list, self.bins)
-                    counts = dict(zip(self.bins, sample_bins.counts))
-
-                results_list.append(self._shannon(counts))
+                results_list.append(
+                    shannon_diversity(
+                        values_list,
+                        self.bins,
+                        categorical=categorical,
+                        categories=categories,
+                    )
+                )
             else:
                 results_list.append(np.nan)
 
         self.series = pd.Series(results_list, index=gdf.index)
 
-    def _shannon(self, data):
-        """ Given a hash { 'species': count } , returns the SDI
 
-        >>> _shannon({'a': 10, 'b': 20, 'c': 30,})
-        1.0114042647073518
+def shannon_diversity(data, bins=None, categorical=False, categories=None):
+    """
+    Calculates the Shannon\'s diversity index of data. Helper function for
+    :py:class:`momepy.Shannon`.
 
-        https://gist.github.com/audy/783125
-        """
+    .. math::
 
-        from math import log as ln
+        \\lambda=\\sum_{i=1}^{R} p_{i}^{2}
 
-        def p(n, N):
-            """ Relative abundance """
-            if n == 0:
-                return 0
-            return (float(n) / N) * ln(float(n) / N)
+    Formula adapted from https://gist.github.com/audy/783125
+    
+    Parameters
+    ----------
+    data : GeoDataFrame
+        GeoDataFrame containing morphological tessellation
+    bins : array, optional
+        array of top edges of classification bins. Result of binnng.bins.
+    categorical : bool (default False)
+        treat values as categories (will not use ``bins``)
+    categories : list-like (default None)
+        list of categories
 
-        N = sum(data.values())
+    Returns
+    -------
+    float
+        Shannon's diversity index
+    
+    See also
+    --------
+    momepy.Shannon : Calculates the Shannon's diversity index of values within neighbours
+    momepy.Simpson : Calculates the Simpson's diversity index of values within neighbours
+    momepy.simpson_diversity : Calculates the Simpson's diversity index of data
+    """
+    from math import log as ln
 
-        return -sum(p(n, N) for n in data.values() if n != 0)
+    if not categorical:
+        try:
+            import mapclassify as mc
+        except ImportError:
+            raise ImportError("The 'mapclassify' package is required")
+
+    def p(n, N):
+        """ Relative abundance """
+        if n == 0:
+            return 0
+        return (float(n) / N) * ln(float(n) / N)
+
+    if categorical:
+        counts = data.value_counts().to_dict()
+        for c in categories:
+            if c not in counts.keys():
+                counts[c] = 0
+    else:
+        sample_bins = mc.UserDefined(data, bins)
+        counts = dict(zip(bins, sample_bins.counts))
+
+    N = sum(counts.values())
+
+    return -sum(p(n, N) for n in counts.values() if n != 0)
 
 
 class Unique:
     """
-    Calculates the number of unique values within neighbours defined in `spatial_weights`.
+    Calculates the number of unique values within neighbours defined in ``spatial_weights``.
 
     .. math::
 
@@ -636,11 +724,13 @@ class Unique:
     gdf : GeoDataFrame
         GeoDataFrame containing morphological tessellation
     values : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored character value.
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored character value.
     spatial_weights : libpysal.weights
         spatial weights matrix
     unique_id : str
-        name of the column with unique id used as spatial_weights index
+        name of the column with unique id used as ``spatial_weights`` index
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -655,20 +745,14 @@ class Unique:
     id : Series
         Series containing used unique ID
 
-
-    References
-    ----------
-
     Examples
     --------
     >>> sw = momepy.sw_high(k=3, gdf=tessellation_df, ids='uID')
     >>> tessellation_df['cluster_unique'] = mm.Unique(tessellation_df, 'cluster', sw, 'uID').series
     100%|██████████| 144/144 [00:00<00:00, 722.50it/s]
-
-
     """
 
-    def __init__(self, gdf, values, spatial_weights, unique_id):
+    def __init__(self, gdf, values, spatial_weights, unique_id, verbose=True):
         self.gdf = gdf
         self.sw = spatial_weights
         self.id = gdf[unique_id]
@@ -683,7 +767,7 @@ class Unique:
         data = data.set_index(unique_id)[values]
 
         results_list = []
-        for index in tqdm(data.index, total=data.shape[0]):
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index].copy()
                 if neighbours:
