@@ -4,24 +4,28 @@
 # elements.py
 # generating derived elements (street edge, block)
 import operator
+from distutils.version import LooseVersion
 
 import geopandas as gpd
+import libpysal
 import numpy as np
 import pandas as pd
 import shapely
-import libpysal
 from scipy.spatial import Voronoi
 from shapely.geometry import MultiPolygon, Point, Polygon
 from shapely.wkt import loads
 from tqdm import tqdm
 
+GPD_08 = str(gpd.__version__) >= LooseVersion("0.8.0")
 
 __all__ = ["buffered_limit", "Tessellation", "Blocks", "get_network_id", "get_node_id"]
 
 
 def buffered_limit(gdf, buffer=100):
     """
-    Define limit for tessellation as a buffer around buildings.
+    Define limit for :class:`momepy.Tessellation` as a buffer around buildings.
+
+    See :cite:`fleischmann2020` for details.
 
     Parameters
     ----------
@@ -50,8 +54,22 @@ def buffered_limit(gdf, buffer=100):
 
 class Tessellation:
     """
-    Generate morphological tessellation around given buildings or proximity bands around given
-    street network.
+    Generate morphological tessellation around given buildings or proximity bands around
+    given street network.
+
+    See :cite:`fleischmann2020` for details of implementation.
+
+    Tessellation requires data of relatively high level of precision and there are three
+    particular patterns causign issues.\n
+    1. Features will collapse into empty polygon - these do not have tessellation
+    cell in the end.\n
+    2. Features will split into MultiPolygon - at some cases, features with narrow links
+    between parts split into two during 'shrinking'. In most cases that is not an issue
+    and resulting tessellation is correct anyway, but sometimes this result in a cell
+    being MultiPolygon, which is not correct.\n
+    3. Overlapping features - features which overlap even after 'shrinking' cause invalid
+    tessellation geometry.\n
+    All three types can be tested prior :class:`momepy.Tessellation` using :class:`momepy.CheckTessellationInput`.
 
     Parameters
     ----------
@@ -64,7 +82,9 @@ class Tessellation:
     shrink : float (default 0.4)
         distance for negative buffer to generate space between adjacent polygons (if geometry type of gdf is (Multi)Polygon).
     segment : float (default 0.5)
-        maximum distance between points after discretisation
+        maximum distance between points after discretization
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -82,6 +102,10 @@ class Tessellation:
         used segment value
     sindex : rtree spatial index
         spatial index of tessellation
+    collapsed : list
+        list of unique_id's of collapsed features (if there are some)
+    multipolygons : list
+        list of unique_id's of features causing MultiPolygons (if there are some)
 
 
     Examples
@@ -112,7 +136,7 @@ class Tessellation:
     queen_corners is currently experimental method only and can cause errors.
     """
 
-    def __init__(self, gdf, unique_id, limit, shrink=0.4, segment=0.5):
+    def __init__(self, gdf, unique_id, limit, shrink=0.4, segment=0.5, verbose=True):
         self.gdf = gdf
         self.id = gdf[unique_id]
         self.limit = limit
@@ -126,22 +150,20 @@ class Tessellation:
             xoff=-centre[0], yoff=-centre[1]
         )
 
-        polys = ["Polygon", "MultiPolygon"]
-        print("Inward offset...")
-        objects["geometry"] = objects.geometry.apply(
-            lambda g: g.buffer(-shrink, cap_style=2, join_style=2)
-            if g.type in polys
-            else g
+        print("Inward offset...") if verbose else None
+        mask = objects.type.isin(["Polygon", "MultiPolygon"])
+        objects.loc[mask, "geometry"] = objects[mask].buffer(
+            -shrink, cap_style=2, join_style=2
         )
 
         objects = objects.explode()
-        objects.reset_index(inplace=True, drop=True)
+        objects = objects.set_index(unique_id)
 
-        print("Discretization...")
+        print("Discretization...") if verbose else None
         objects["geometry"] = objects["geometry"].apply(self._densify, segment=segment)
 
-        print("Generating input point array...")
-        points, ids = self._point_array(objects, unique_id)
+        print("Generating input point array...") if verbose else None
+        points, ids = self._point_array(objects, verbose=verbose)
 
         # add convex hull buffered large distance to eliminate infinity issues
         series = gpd.GeoSeries(limit, crs=gdf.crs).translate(
@@ -154,13 +176,15 @@ class Tessellation:
             points.append(hull_array[i])
             ids.append(-1)
 
-        print("Generating Voronoi diagram...")
+        print("Generating Voronoi diagram...") if verbose else None
         voronoi_diagram = Voronoi(np.array(points))
 
-        print("Generating GeoDataFrame...")
-        regions_gdf = self._regions(voronoi_diagram, unique_id, ids, crs=gdf.crs)
+        print("Generating GeoDataFrame...") if verbose else None
+        regions_gdf = self._regions(
+            voronoi_diagram, unique_id, ids, crs=gdf.crs, verbose=verbose
+        )
 
-        print("Dissolving Voronoi polygons...")
+        print("Dissolving Voronoi polygons...") if verbose else None
         morphological_tessellation = regions_gdf[[unique_id, "geometry"]].dissolve(
             by=unique_id, as_index=False
         )
@@ -170,7 +194,7 @@ class Tessellation:
         ].translate(xoff=centre[0], yoff=centre[1])
 
         morphological_tessellation, sindex = self._cut(
-            morphological_tessellation, limit, unique_id
+            morphological_tessellation, limit, unique_id, verbose=verbose
         )
         self.sindex = sindex
 
@@ -180,7 +204,18 @@ class Tessellation:
 
     def queen_corners(self, sensitivity):
         """
-        Experimental: Fix unprecise corners.
+        Experimental function to fix unprecise corners.
+
+        Parameters
+        ----------
+        sensitivity : float
+            maximum distace between two corners to be merged
+
+        Returns
+        -------
+        tessellation : GeoDataFrame
+            GeoDataFrame containing resulting tessellation with fixed corners
+
         """
         tessellation = self.tessellation.copy()
         changes = {}
@@ -268,9 +303,9 @@ class Tessellation:
         """
         Returns centre coords of gdf.
         """
-        bounds = gdf["geometry"].bounds
-        centre_x = (bounds["maxx"].max() + bounds["minx"].min()) / 2
-        centre_y = (bounds["maxy"].max() + bounds["miny"].min()) / 2
+        bounds = gdf.total_bounds
+        centre_x = (bounds[0] + bounds[2]) / 2
+        centre_y = (bounds[1] + bounds[3]) / 2
         return centre_x, centre_y
 
     # densify geometry before Voronoi tesselation
@@ -278,6 +313,7 @@ class Tessellation:
         """
         Returns densified geoemtry with segments no longer than `segment`.
         """
+        # TODO: use pygeos interpolate once in shapely
         # temporary solution for readthedocs fail. - cannot mock osgeo
         try:
             from osgeo import ogr
@@ -298,36 +334,35 @@ class Tessellation:
         except Exception:
             return poly
 
-    def _point_array(self, objects, unique_id):
+    def _point_array(self, objects, verbose):
         """
-        Returns lists of points and ids based on geometry and unique_id.
+        Returns lists of points and ids based on geometry and index.
         """
         points = []
         ids = []
-        for idx, row in tqdm(objects.iterrows(), total=objects.shape[0]):
-            if row["geometry"].type in ["Polygon", "MultiPolygon"]:
-                poly_ext = row["geometry"].boundary
-            else:
-                poly_ext = row["geometry"]
-            if poly_ext is not None:
-                if poly_ext.type == "MultiLineString":
-                    for line in poly_ext:
-                        point_coords = line.coords
-                        row_array = np.array(point_coords[:-1]).tolist()
-                        for i, a in enumerate(row_array):
-                            points.append(row_array[i])
-                            ids.append(row[unique_id])
-                elif poly_ext.type == "LineString":
-                    point_coords = poly_ext.coords
+        mask = objects.type.isin(["Polygon", "MultiPolygon"])
+        objects.loc[mask, "geometry"] = objects[mask].boundary
+        for idx, poly_ext in tqdm(
+            objects.geometry.iteritems(), total=objects.shape[0], disable=not verbose
+        ):
+            if poly_ext.type == "MultiLineString":
+                for line in poly_ext.geoms:
+                    point_coords = line.coords
                     row_array = np.array(point_coords[:-1]).tolist()
                     for i, a in enumerate(row_array):
                         points.append(row_array[i])
-                        ids.append(row[unique_id])
-                else:
-                    raise Exception("Boundary type is {}".format(poly_ext.type))
+                        ids.append(idx)
+            elif poly_ext.type == "LineString":
+                point_coords = poly_ext.coords
+                row_array = np.array(point_coords[:-1]).tolist()
+                for i, a in enumerate(row_array):
+                    points.append(row_array[i])
+                    ids.append(idx)
+            else:
+                raise Exception("Boundary type is {}".format(poly_ext.type))
         return points, ids
 
-    def _regions(self, voronoi_diagram, unique_id, ids, crs):
+    def _regions(self, voronoi_diagram, unique_id, ids, crs, verbose):
         """
         Generate GeoDataFrame of Voronoi regions from scipy.spatial.Voronoi.
         """
@@ -344,7 +379,9 @@ class Tessellation:
 
         # convert vertices to Polygons
         polygons = []
-        for region in tqdm(regions.vertices, desc="Vertices to Polygons"):
+        for region in tqdm(
+            regions.vertices, desc="Vertices to Polygons", disable=not verbose
+        ):
             if -1 not in region:
                 polygons.append(Polygon(voronoi_diagram.vertices[region]))
             else:
@@ -363,52 +400,64 @@ class Tessellation:
         regions_gdf.crs = crs
         return regions_gdf
 
-    def _cut(self, tessellation, limit, unique_id):
+    def _cut(self, tessellation, limit, unique_id, verbose):
         """
         Cut tessellation by the limit (Multi)Polygon.
 
         ADD: add option to delete everything outside of limit. Now it keeps it.
         """
-        print("Preparing limit for edge resolving...")
+        print("Preparing limit for edge resolving...") if verbose else None
         geometry_cut = _split_lines(limit, 100)
 
-        print("Building R-tree...")
+        print("Building R-tree...") if verbose else None
         sindex = tessellation.sindex
         # find the points that intersect with each subpolygon and add them to points_within_geometry
-        print("Identifying edge cells...")
+        print("Identifying edge cells...") if verbose else None
         to_cut = pd.DataFrame()
-        for poly in tqdm(geometry_cut, total=(len(geometry_cut))):
-            # find approximate matches with r-tree, then precise matches from those approximate ones
-            possible_matches_index = list(sindex.intersection(poly.bounds))
-            possible_matches = tessellation.iloc[possible_matches_index]
-            precise_matches = possible_matches[possible_matches.intersects(poly)]
-            to_cut = to_cut.append(precise_matches)
+        if GPD_08:
+            inp, tree = sindex.query_bulk(
+                gpd.array.from_shapely([g for g in geometry_cut.geoms]),
+                predicate="intersects",
+            )
+            subselection = list(tessellation.iloc[list(set(tree))].index)
+        else:
+            for poly in tqdm(
+                geometry_cut, total=(len(geometry_cut)), disable=not verbose
+            ):
+                # find approximate matches with r-tree, then precise matches from those approximate ones
+                possible_matches_index = list(sindex.intersection(poly.bounds))
+                possible_matches = tessellation.iloc[possible_matches_index]
+                precise_matches = possible_matches[possible_matches.intersects(poly)]
+                to_cut = to_cut.append(precise_matches)
 
-        # delete duplicates
-        to_cut = to_cut.drop_duplicates(subset=[unique_id])
-        subselection = list(to_cut.index)
+            # delete duplicates
+            to_cut = to_cut.drop_duplicates(subset=[unique_id])
+            subselection = list(to_cut.index)
 
-        print("Cutting...")
-        for idx, rgeom in tqdm(
-            tessellation.loc[subselection].geometry.iteritems(),
-            total=tessellation.loc[subselection].shape[0],
-        ):
-            intersection = rgeom.intersection(limit)
-            if intersection.type == "MultiPolygon":
-                areas = {}
-                for p, i in enumerate(intersection):
-                    area = intersection[p].area
-                    areas[p] = area
-                maximal = max(areas.items(), key=operator.itemgetter(1))[0]
-                tessellation.loc[idx, "geometry"] = intersection[maximal]
-            elif intersection.type == "GeometryCollection":
-                for geom in list(intersection.geoms):
-                    if geom.type != "Polygon":
-                        pass
-                    else:
-                        tessellation.loc[idx, "geometry"] = geom
-            else:
-                tessellation.loc[idx, "geometry"] = intersection
+        print("Cutting...") if verbose else None
+        tessellation.loc[subselection, "geometry"] = tessellation.loc[
+            subselection
+        ].intersection(limit)
+        mask = tessellation.type.isin(["MultiPolygon", "GeometryCollection"])
+        if mask.any():
+            for idx, intersection in tqdm(
+                tessellation[mask].geometry.iteritems(),
+                total=len(tessellation[mask]),
+                disable=not verbose,
+            ):
+                if intersection.type == "MultiPolygon":
+                    areas = {}
+                    for p, i in enumerate(intersection):
+                        area = intersection[p].area
+                        areas[p] = area
+                    maximal = max(areas.items(), key=operator.itemgetter(1))[0]
+                    tessellation.loc[idx, "geometry"] = intersection[maximal]
+                elif intersection.type == "GeometryCollection":
+                    for geom in list(intersection.geoms):
+                        if geom.type != "Polygon":
+                            pass
+                        else:
+                            tessellation.loc[idx, "geometry"] = geom
         return tessellation, sindex
 
     def _check_result(self, tesselation, orig_gdf, unique_id):
@@ -421,28 +470,33 @@ class Tessellation:
         if len(ids_original) != len(ids_generated):
             import warnings
 
-            diff = set(ids_original).difference(ids_generated)
+            self.collapsed = set(ids_original).difference(ids_generated)
             warnings.warn(
                 "Tessellation does not fully match buildings. {len} element(s) collapsed "
-                "during generation - unique_id: {i}".format(len=len(diff), i=diff)
+                "during generation - unique_id: {i}".format(
+                    len=len(self.collapsed), i=self.collapsed
+                )
             )
 
         # check MultiPolygons - usually caused by error in input geometry
-        uids = tesselation[tesselation.geometry.type == "MultiPolygon"][unique_id]
-        if len(uids) > 0:
+        self.multipolygons = tesselation[tesselation.geometry.type == "MultiPolygon"][
+            unique_id
+        ]
+        if len(self.multipolygons) > 0:
             import warnings
 
             warnings.warn(
                 "Tessellation contains MultiPolygon elements. Initial objects should be edited. "
-                "unique_id of affected elements: {}".format(list(uids))
+                "unique_id of affected elements: {}".format(list(self.multipolygons))
             )
 
 
 class Blocks:
     """
-    Generate blocks based on buildings, tesselation and street network
+    Generate blocks based on buildings, tesselation and street network.
 
-    Captures bID to buildings and tesselation as attributes.
+    Dissolves tessellation cells based on street-network based polygons.
+    Links resulting id to ``buildings`` and ``tesselation`` as attributes.
 
     Parameters
     ----------
@@ -455,8 +509,10 @@ class Blocks:
     id_name : str
         name of the unique blocks id column to be generated
     unique_id : str
-        name of the column with unique id. If there is none, it could be generated by unique_id().
+        name of the column with unique id. If there is none, it could be generated by :func:`momepy.unique_id`.
         This should be the same for cells and buildings, id's should match.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -503,7 +559,9 @@ class Blocks:
 
     """
 
-    def __init__(self, tessellation, edges, buildings, id_name, unique_id):
+    def __init__(
+        self, tessellation, edges, buildings, id_name, unique_id, verbose=True
+    ):
         self.tessellation = tessellation
         self.edges = edges
         self.buildings = buildings
@@ -517,25 +575,30 @@ class Blocks:
 
         cells_copy = tessellation[[unique_id, "geometry"]].copy()
 
-        print("Buffering streets...")
+        print("Buffering streets...") if verbose else None
         street_buff = edges.copy()
         street_buff["geometry"] = street_buff.buffer(0.1)
 
-        print("Generating spatial index...")
+        print("Generating spatial index...") if verbose else None
         streets_index = street_buff.sindex
 
-        print("Difference...")
+        print("Difference...") if verbose else None
         new_geom = []
 
         for ix, cell in tqdm(
-            cells_copy.geometry.iteritems(), total=cells_copy.shape[0]
+            cells_copy.geometry.iteritems(),
+            total=cells_copy.shape[0],
+            disable=not verbose,
         ):
-            possible_matches_index = list(streets_index.intersection(cell.bounds))
+            if GPD_08:
+                possible_matches_index = streets_index.query(cell)
+            else:
+                possible_matches_index = list(streets_index.intersection(cell.bounds))
             possible_matches = street_buff.iloc[possible_matches_index]
-            new_geom.append(cell.difference(possible_matches.geometry.unary_union))
+            new_geom.append(cell.difference(possible_matches.unary_union))
 
-        print("Defining adjacency...")
-        blocks_gdf = gpd.GeoDataFrame(geometry=gpd.GeoSeries(new_geom))
+        print("Defining adjacency...") if verbose else None
+        blocks_gdf = gpd.GeoDataFrame(geometry=new_geom)
         blocks_gdf = blocks_gdf.explode().reset_index(drop=True)
 
         spatial_weights = libpysal.weights.Queen.from_dataframe(
@@ -544,7 +607,9 @@ class Blocks:
 
         patches = {}
         jID = 1
-        for idx in tqdm(blocks_gdf.index, total=blocks_gdf.shape[0]):
+        for idx in tqdm(
+            blocks_gdf.index, total=blocks_gdf.shape[0], disable=not verbose
+        ):
 
             # if the id is already present in courtyards, continue (avoid repetition)
             if idx in patches:
@@ -552,55 +617,51 @@ class Blocks:
             else:
                 to_join = [idx]  # list of indices which should be joined together
                 neighbours = []  # list of neighbours
-                weights = spatial_weights.neighbors[
+                neighbours += spatial_weights.neighbors[
                     idx
                 ]  # neighbours from spatial weights
-                for w in weights:
-                    neighbours.append(w)  # make a list from weigths
 
                 for n in neighbours:
                     while (
                         n not in to_join
                     ):  # until there is some neighbour which is not in to_join
                         to_join.append(n)
-                        weights = spatial_weights.neighbors[n]
-                        for w in weights:
-                            neighbours.append(
-                                w
-                            )  # extend neighbours by neighbours of neighbours :)
+                        neighbours += spatial_weights.neighbors[
+                            n
+                        ]  # extend neighbours by neighbours of neighbours :)
                 for b in to_join:
                     patches[b] = jID  # fill dict with values
                 jID = jID + 1
 
         blocks_gdf["patch"] = blocks_gdf.index.map(patches)
 
-        print("Defining street-based blocks...")
+        print("Defining street-based blocks...") if verbose else None
         blocks_single = blocks_gdf.dissolve(by="patch")
         blocks_single.crs = buildings.crs
 
         blocks_single["geometry"] = blocks_single.buffer(0.1)
 
-        print("Defining block ID...")  # street based
+        print("Defining block ID...") if verbose else None  # street based
         blocks_single[id_name] = range(len(blocks_single))
 
-        print("Generating centroids...")
+        print("Generating centroids...") if verbose else None
         buildings_c = buildings.copy()
         buildings_c["geometry"] = buildings_c.representative_point()  # make points
 
-        print("Spatial join...")
+        print("Spatial join...") if verbose else None
         centroids_tempID = gpd.sjoin(
             buildings_c, blocks_single, how="left", op="intersects"
         )
 
         tempID_to_uID = centroids_tempID[[unique_id, id_name]]
 
-        print("Attribute join (tesselation)...")
+        print("Attribute join (tesselation)...") if verbose else None
         cells_copy = cells_copy.merge(tempID_to_uID, on=unique_id, how="left")
 
-        print("Generating blocks...")
+        print("Generating blocks...") if verbose else None
         blocks = cells_copy.dissolve(by=id_name)
 
-        print("Multipart to singlepart...")
+        print("Multipart to singlepart...") if verbose else None
         blocks = blocks.explode()
         blocks.reset_index(inplace=True, drop=True)
 
@@ -611,17 +672,25 @@ class Blocks:
 
         # if polygon is within another one, delete it
         sindex = blocks.sindex
-        for idx, geom in tqdm(blocks.geometry.iteritems(), total=blocks.shape[0]):
-            possible_matches = list(sindex.intersection(geom.bounds))
-            possible_matches.remove(idx)
-            possible = blocks.iloc[possible_matches]
+        if not GPD_08:
+            for idx, geom in tqdm(
+                blocks.geometry.iteritems(), total=blocks.shape[0], disable=not verbose
+            ):
+                possible_matches = list(sindex.intersection(geom.bounds))
+                possible_matches.remove(idx)
+                possible = blocks.iloc[possible_matches]
 
-            for geom2 in possible.geometry:
-                if geom.within(geom2):
-                    blocks.loc[idx, "delete"] = 1
-
-        if "delete" in blocks.columns:
-            blocks = blocks.drop(list(blocks.loc[blocks["delete"] == 1].index))
+                for geom2 in possible.geometry:
+                    if geom.within(geom2):
+                        blocks.loc[idx, "delete"] = 1
+            if "delete" in blocks.columns:
+                blocks = blocks.drop(list(blocks.loc[blocks["delete"] == 1].index))
+        else:
+            inp, res = sindex.query_bulk(blocks.geometry, predicate="within")
+            res = res[~(inp == res)]
+            mask = np.ones(len(blocks.index), dtype=bool)
+            mask[list(set(res))] = False
+            blocks = blocks.loc[mask]
 
         self.blocks = blocks[[id_name, "geometry"]]
 
@@ -630,20 +699,20 @@ class Blocks:
         )
         bl_ID_to_uID = centroids_w_bl_ID2[[unique_id, id_name]]
 
-        print("Attribute join (buildings)...")
+        print("Attribute join (buildings)...") if verbose else None
         buildings_m = buildings[[unique_id]].merge(
             bl_ID_to_uID, on=unique_id, how="left"
         )
         self.buildings_id = buildings_m[id_name]
 
-        print("Attribute join (tesselation)...")
+        print("Attribute join (tesselation)...") if verbose else None
         cells_m = tessellation[[unique_id]].merge(
             bl_ID_to_uID, on=unique_id, how="left"
         )
         self.tessellation_id = cells_m[id_name]
 
 
-def get_network_id(left, right, network_id, min_size=100):
+def get_network_id(left, right, network_id, min_size=100, verbose=True):
     """
     Snap each element (preferably building) to the closest street network segment, saves its id.
 
@@ -655,13 +724,15 @@ def get_network_id(left, right, network_id, min_size=100):
         GeoDataFrame containing objects to snap
     right : GeoDataFrame
         GeoDataFrame containing street network with unique network ID.
-        If there is none, it could be generated by :py:func:`momepy.elements.unique_id`.
+        If there is none, it could be generated by :func:`momepy.unique_id`.
     network_id : str, list, np.array, pd.Series (default None)
-        the name of the streets dataframe column, np.array, or pd.Series with network unique id.
+        the name of the streets dataframe column, ``np.array``, or ``pd.Series`` with network unique id.
     min_size : int (default 100)
         min_size should be a vaule such that if you build a box centered in each
-        building centroid with edges of size `2*min_size`, you know a priori that at least one
+        building centroid with edges of size ``2*min_size``, you know a priori that at least one
         segment is intersected with the box.
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Returns
     -------
@@ -678,11 +749,6 @@ def get_network_id(left, right, network_id, min_size=100):
     1
     """
     INFTY = 1000000000000
-    MIN_SIZE = min_size
-    # MIN_SIZE should be a vaule such that if you build a box centered in each
-    # point with edges of size 2*MIN_SIZE, you know a priori that at least one
-    # segment is intersected with the box. Otherwise, you could get an inexact
-    # solution, there is an exception checking this, though.
     left = left.copy()
     right = right.copy()
 
@@ -690,17 +756,23 @@ def get_network_id(left, right, network_id, min_size=100):
         right["mm_nid"] = network_id
         network_id = "mm_nid"
 
-    print("Generating centroids...")
+    print("Generating centroids...") if verbose else None
     buildings_c = left.copy()
 
     buildings_c["geometry"] = buildings_c.centroid  # make centroids
 
-    print("Generating rtree...")
+    print("Generating rtree...") if verbose else None
     idx = right.sindex
 
+    # TODO: use sjoin nearest once done
     result = []
-    for p in tqdm(buildings_c.geometry, total=buildings_c.shape[0], desc="Snapping"):
-        pbox = (p.x - MIN_SIZE, p.y - MIN_SIZE, p.x + MIN_SIZE, p.y + MIN_SIZE)
+    for p in tqdm(
+        buildings_c.geometry,
+        total=buildings_c.shape[0],
+        desc="Snapping",
+        disable=not verbose,
+    ):
+        pbox = (p.x - min_size, p.y - min_size, p.x + min_size, p.y + min_size)
         hits = list(idx.intersection(pbox))
         d = INFTY
         nid = None
@@ -726,11 +798,11 @@ def get_network_id(left, right, network_id, min_size=100):
     return series
 
 
-def get_node_id(objects, nodes, edges, node_id, edge_id):
+def get_node_id(objects, nodes, edges, node_id, edge_id, verbose=True):
     """
     Snap each building to closest street network node on the closest network edge.
 
-    Adds node ID to objects (preferably buildings). Gets ID of edge (:py:func:`momepy.get_node_id`)
+    Adds node ID to objects (preferably buildings). Gets ID of edge (:func:`momepy.get_network_id`)
     , and determines which of its end points is closer to building centroid.
 
     Parameters
@@ -739,12 +811,14 @@ def get_node_id(objects, nodes, edges, node_id, edge_id):
         GeoDataFrame containing objects to snap
     nodes : GeoDataFrame
         GeoDataFrame containing street nodes with unique node ID.
-        If there is none, it could be generated by :py:func:`momepy.unique_id`.
+        If there is none, it could be generated by :func:`momepy.unique_id`.
     edges : GeoDataFrame
         GeoDataFrame containing street edges with unique edge ID and IDs of start
-        and end points of each segment. Start and endpoints are default outcome of :py:func:`momepy.nx_to_gdf`.
+        and end points of each segment. Start and endpoints are default outcome of :func:`momepy.nx_to_gdf`.
     node_id : str, list, np.array, pd.Series (default None)
-        the name of the nodes dataframe column, np.array, or pd.Series with unique id
+        the name of the nodes dataframe column, ``np.array``, or ``pd.Series`` with unique id
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Returns
     -------
@@ -752,27 +826,26 @@ def get_node_id(objects, nodes, edges, node_id, edge_id):
         Series containing node ID for objects
 
     """
-    nodes = nodes.copy()
+    nodes = nodes.set_index(node_id)
+    edges = edges.set_index(edge_id)
     if not isinstance(node_id, str):
         nodes["mm_noid"] = node_id
         node_id = "mm_noid"
 
     results_list = []
-    for row in tqdm(
-        objects[[edge_id, objects._geometry_column_name]].itertuples(),
-        total=objects.shape[0],
+    centroids = objects.centroid
+    for eid, centroid in tqdm(
+        zip(objects[edge_id], centroids), total=objects.shape[0], disable=not verbose
     ):
-        if np.isnan(row[1]):
-
+        if np.isnan(eid):
             results_list.append(np.nan)
         else:
-            centroid = row[2].centroid
-            edge = edges.loc[edges[edge_id] == row[1]].iloc[0]
+            edge = edges.loc[eid]
             startID = edge.node_start
-            start = nodes.loc[nodes[node_id] == startID].iloc[0].geometry
+            start = nodes.loc[startID].geometry
             sd = centroid.distance(start)
             endID = edge.node_end
-            end = nodes.loc[nodes[node_id] == endID].iloc[0].geometry
+            end = nodes.loc[endID].geometry
             ed = centroid.distance(end)
             if sd > ed:
                 results_list.append(endID)
@@ -788,6 +861,7 @@ def _split_lines(polygon, distance):
     list_points = []
     current_dist = distance  # set the current distance to place the point
 
+    # TODO: use pygeos interpolate
     boundary = polygon.boundary  # make shapely MultiLineString object
     if boundary.type == "LineString":
         line_length = boundary.length  # get the total length of the line
