@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # shape.py
-# definitons of shape characters
+# definitions of shape characters
 
 import math
 import random
@@ -18,7 +18,7 @@ __all__ = [
     "VolumeFacadeRatio",
     "CircularCompactness",
     "SquareCompactness",
-    "Convexeity",
+    "Convexity",
     "CourtyardIndex",
     "Rectangularity",
     "ShapeIndex",
@@ -34,20 +34,22 @@ __all__ = [
 
 class FormFactor:
     """
-    Calculates form factor of each object in given geoDataFrame.
+    Calculates form factor of each object in given GeoDataFrame.
 
     .. math::
         area \\over {volume^{2 \\over 3}}
 
+    Adapted from :cite:`bourdic2012`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     volumes : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored volume value.
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored volume value.
         (To calculate volume you can use :py:func:`momepy.volume`)
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -61,12 +63,6 @@ class FormFactor:
     areas : Series
         Series containing used area values
 
-    References
-    ----------
-    Bourdic, L., Salat, S. and Nowacki, C. (2012) ‘Assessing cities: a new system
-    of cross-scale spatial indicators’, Building Research & Information, 40(5),
-    pp. 592–605. doi: 10.1080/09613218.2012.703488self.
-
     Examples
     --------
     >>> buildings_df['formfactor'] = momepy.FormFactor(buildings_df, 'volume').series
@@ -102,20 +98,22 @@ class FormFactor:
 
 class FractalDimension:
     """
-    Calculates fractal dimension of each object in given geoDataFrame.
+    Calculates fractal dimension of each object in given GeoDataFrame.
 
     .. math::
         {2log({{perimeter} \\over {4}})} \\over log(area)
 
+    Based on :cite:`mcgarigal1995fragstats`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
     perimeters : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored perimeter value. If set to None, function will calculate perimeters
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored perimeter value. If set to ``None``, function will calculate perimeters
         during the process without saving them separately.
 
     Attributes
@@ -129,14 +127,9 @@ class FractalDimension:
     areas : Series
         Series containing used area values
 
-    References
-    ----------
-    McGarigal, K., & Marks, B. (1995). FRAGSTATS: spatial pattern analysis program for quantifying landscape structure.
-
     Examples
     --------
     >>> buildings_df['fractal'] = momepy.FractalDimension(buildings_df, 'area', 'peri').series
-    100%|██████████| 144/144 [00:00<00:00, 3928.09it/s]
     >>> buildings_df.fractal[0]
     1.0726778567038908
     """
@@ -167,21 +160,23 @@ class FractalDimension:
 
 class VolumeFacadeRatio:
     """
-    Calculates volume/facade ratio of each object in given geoDataFrame.
+    Calculates volume/facade ratio of each object in given GeoDataFrame.
 
     .. math::
-        volume \\over perimeter * heigth
+        volume \\over perimeter * height
+
+    Adapted from :cite:`schirmer2015`.
 
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     heights : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored height value
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored height value
     volumes : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored volume value
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored volume value
     perimeters : , list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored perimeter value
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored perimeter value
 
 
     Attributes
@@ -195,14 +190,8 @@ class VolumeFacadeRatio:
     volumes : Series
         Series containing used volume values
 
-    References
-    ----------
-    Schirmer, P. M. and Axhausen, K. W. (2015) ‘A multiscale classification of
-    urban morphology’, Journal of Transport and Land Use, 9(1), pp. 101–130.
-    doi: 10.5198/jtlu.2015.667.
-
     Examples
-    -----
+    --------
     >>> buildings_df['vfr'] = momepy.VolumeFacadeRatio(buildings_df, 'height').series
     >>> buildings_df.vfr[0]
     5.310715735236504
@@ -406,17 +395,19 @@ def _circle_area(points):
 
 class CircularCompactness:
     """
-    Calculates compactness index of each object in given geoDataFrame.
+    Calculates compactness index of each object in given GeoDataFrame.
 
     .. math::
         area \\over \\textit{area of enclosing circle}
 
+    Adapted from :cite:`dibble2017`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -428,15 +419,9 @@ class CircularCompactness:
     areas : Series
         Series containing used area values
 
-    References
-    ----------
-    Dibble, J. (2016) Urban Morphometrics: Towards a Quantitative Science of Urban
-    Form. University of Strathclyde.
-
     Examples
     --------
     >>> buildings_df['circ_comp'] = momepy.CircularCompactness(buildings_df, 'area').series
-    100%|██████████| 144/144 [00:00<00:00, 2498.75it/s]
     >>> buildings_df['circ_comp'][0]
     0.572145421828038
     """
@@ -460,22 +445,24 @@ class CircularCompactness:
 
 class SquareCompactness:
     """
-    Calculates compactness index of each object in given geoDataFrame.
+    Calculates compactness index of each object in given GeoDataFrame.
 
     .. math::
         \\begin{equation*}
         \\left(\\frac{4 \\sqrt{area}}{perimeter}\\right) ^ 2
         \\end{equation*}
 
+    Adapted from :cite:`feliciotti2018`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
     perimeters : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored perimeter value. If set to None, function will calculate perimeters
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored perimeter value. If set to ``None``, function will calculate perimeters
         during the process without saving them separately.
 
     Attributes
@@ -489,11 +476,6 @@ class SquareCompactness:
     perimeters : Series
         Series containing used perimeter values
 
-    References
-    ----------
-    Feliciotti A (2018) RESILIENCE AND URBAN DESIGN:A SYSTEMS APPROACH TO THE
-    STUDY OF RESILIENCE IN URBAN FORM. LEARNING FROM THE CASE OF GORBALS. Glasgow.
-
     Examples
     --------
     >>> buildings_df['squ_comp'] = momepy.SquareCompactness(buildings_df).series
@@ -524,19 +506,21 @@ class SquareCompactness:
         self.series = ((np.sqrt(gdf[areas]) * 4) / gdf[perimeters]) ** 2
 
 
-class Convexeity:
+class Convexity:
     """
-    Calculates convexeity index of each object in given geoDataFrame.
+    Calculates Convexity index of each object in given GeoDataFrame.
 
     .. math::
         area \\over \\textit{convex hull area}
 
+    Adapted from :cite:`dibble2017`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -548,15 +532,10 @@ class Convexeity:
     areas : Series
         Series containing used area values
 
-    References
-    ----------
-    Dibble, J. (2016) Urban Morphometrics: Towards a Quantitative Science of Urban
-    Form. University of Strathclyde.
-
     Examples
     --------
-    >>> buildings_df['convexeity'] = momepy.Convexeity(buildings_df).series
-    >>> buildings_df.convexeity[0]
+    >>> buildings_df['convexity'] = momepy.Convexity(buildings_df).series
+    >>> buildings_df['convexity'][0]
     0.8151964258521672
     """
 
@@ -576,20 +555,22 @@ class Convexeity:
 
 class CourtyardIndex:
     """
-    Calculates courtyard index of each object in given geoDataFrame.
+    Calculates courtyard index of each object in given GeoDataFrame.
 
     .. math::
         \\textit{area of courtyards} \\over \\textit{total area}
 
+    Adapted from :cite:`schirmer2015`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     courtyard_areas : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored area value
-        (To calculate volume you can use :py:func:`momepy.dimension.courtyard_area`)
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value
+        (To calculate volume you can use :py:class:`momepy.CourtyardArea`)
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -603,19 +584,13 @@ class CourtyardIndex:
     areas : Series
         Series containing used area values
 
-    References
-    ----------
-    Schirmer, P. M. and Axhausen, K. W. (2015) ‘A multiscale classification of
-    urban morphology’, Journal of Transport and Land Use, 9(1), pp. 101–130.
-    doi: 10.5198/jtlu.2015.667.
-
     Examples
     --------
     >>> buildings_df['courtyard_index'] = momepy.CourtyardIndex(buildings, 'courtyard_area', 'area').series
     >>> buildings_df.courtyard_index[80]
     0.16605915738643523
 
-    >>> buildings_df['courtyard_index2'] = momepy.CourtyardIndex(buildings_df, momepy.courtyard_area(buildings_df).ca).series
+    >>> buildings_df['courtyard_index2'] = momepy.CourtyardIndex(buildings_df, momepy.CourtyardArea(buildings_df).series).series
     >>> buildings_df.courtyard_index2[80]
     0.16605915738643523
     """
@@ -639,17 +614,19 @@ class CourtyardIndex:
 
 class Rectangularity:
     """
-    Calculates rectangularity of each object in given geoDataFrame.
+    Calculates rectangularity of each object in given GeoDataFrame.
 
     .. math::
         {area \\over \\textit{minimum bounding rotated rectangle area}}
 
+    Adapted from :cite:`dibble2017`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -661,11 +638,6 @@ class Rectangularity:
     areas : Series
         Series containing used area values
 
-    References
-    ----------
-    Dibble, J. (2016) Urban Morphometrics: Towards a Quantitative Science of Urban
-    Form. University of Strathclyde.
-
     Examples
     --------
     >>> buildings_df['rectangularity'] = momepy.Rectangularity(buildings_df, 'area').series
@@ -692,7 +664,7 @@ class Rectangularity:
 
 class ShapeIndex:
     """
-    Calculates shape index of each object in given geoDataFrame.
+    Calculates shape index of each object in given GeoDataFrame.
 
     .. math::
         {\\sqrt{{area} \\over {\\pi}}} \\over {0.5 * \\textit{longest axis}}
@@ -702,9 +674,9 @@ class ShapeIndex:
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     longest_axis : str, list, np.array, pd.Series
-        the name of the dataframe column, np.array, or pd.Series where is stored longest axis value
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored longest axis value
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -747,9 +719,9 @@ class ShapeIndex:
 
 class Corners:
     """
-    Calculates number of corners of each object in given geoDataFrame.
+    Calculates number of corners of each object in given GeoDataFrame.
 
-    Uses only external shape (shapely.geometry.exterior), courtyards are not included.
+    Uses only external shape (``shapely.geometry.exterior``), courtyards are not included.
 
     .. math::
         \\sum corner
@@ -758,6 +730,8 @@ class Corners:
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -777,7 +751,7 @@ class Corners:
 
     """
 
-    def __init__(self, gdf):
+    def __init__(self, gdf, verbose=True):
         self.gdf = gdf
 
         # define empty list for results
@@ -798,7 +772,7 @@ class Corners:
             return False
 
         # fill new column with the value of area, iterating over rows one by one
-        for geom in tqdm(gdf.geometry, total=gdf.shape[0]):
+        for geom in tqdm(gdf.geometry, total=gdf.shape[0], disable=not verbose):
             corners = 0  # define empty variables
             points = list(geom.exterior.coords)  # get points of a shape
             stop = len(points) - 1  # define where to stop
@@ -834,19 +808,23 @@ class Corners:
 
 class Squareness:
     """
-    Calculates squareness of each object in given geoDataFrame.
+    Calculates squareness of each object in given GeoDataFrame.
 
-    Uses only external shape (shapely.geometry.exterior), courtyards are not included.
+    Uses only external shape (``shapely.geometry.exterior``), courtyards are not included.
 
     .. math::
         \\mu=\\frac{\\sum_{i=1}^{N} d_{i}}{N}
 
-    where `d` is the deviation of angle of corner `i` from 90 degrees.
+    where :math:`d` is the deviation of angle of corner :math:`i` from 90 degrees.
+
+    Adapted from :cite:`dibble2017`.
 
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -855,11 +833,6 @@ class Squareness:
     gdf : GeoDataFrame
         original GeoDataFrame
 
-    References
-    ----------
-    Dibble, J. (2016) Urban Morphometrics: Towards a Quantitative Science of Urban
-    Form. University of Strathclyde.
-
     Examples
     --------
     >>> buildings_df['squareness'] = momepy.Squareness(buildings_df).series
@@ -868,7 +841,7 @@ class Squareness:
     3.7075816043359864
     """
 
-    def __init__(self, gdf):
+    def __init__(self, gdf, verbose=True):
         self.gdf = gdf
         # define empty list for results
         results_list = []
@@ -883,62 +856,68 @@ class Squareness:
             return angle
 
         # fill new column with the value of area, iterating over rows one by one
-        for geom in tqdm(gdf.geometry, total=gdf.shape[0]):
-            angles = []
-            points = list(geom.exterior.coords)  # get points of a shape
-            stop = len(points) - 1  # define where to stop
-            for i in np.arange(
-                len(points)
-            ):  # for every point, calculate angle and add 1 if True angle
-                if i == 0:
-                    continue
-                elif i == stop:
-                    a = np.asarray(points[i - 1])
-                    b = np.asarray(points[i])
-                    c = np.asarray(points[1])
-                    ang = _angle(a, b, c)
-
-                    if ang <= 175:
-                        angles.append(ang)
-                    elif _angle(a, b, c) >= 185:
-                        angles.append(ang)
-                    else:
+        for geom in tqdm(gdf.geometry, total=gdf.shape[0], disable=not verbose):
+            if geom.type == "Polygon":
+                angles = []
+                points = list(geom.exterior.coords)  # get points of a shape
+                stop = len(points) - 1  # define where to stop
+                for i in np.arange(
+                    len(points)
+                ):  # for every point, calculate angle and add 1 if True angle
+                    if i == 0:
                         continue
+                    elif i == stop:
+                        a = np.asarray(points[i - 1])
+                        b = np.asarray(points[i])
+                        c = np.asarray(points[1])
+                        ang = _angle(a, b, c)
 
-                else:
-                    a = np.asarray(points[i - 1])
-                    b = np.asarray(points[i])
-                    c = np.asarray(points[i + 1])
-                    ang = _angle(a, b, c)
+                        if ang <= 175:
+                            angles.append(ang)
+                        elif _angle(a, b, c) >= 185:
+                            angles.append(ang)
+                        else:
+                            continue
 
-                    if _angle(a, b, c) <= 175:
-                        angles.append(ang)
-                    elif _angle(a, b, c) >= 185:
-                        angles.append(ang)
                     else:
-                        continue
-            deviations = [abs(90 - i) for i in angles]
-            results_list.append(np.mean(deviations))
+                        a = np.asarray(points[i - 1])
+                        b = np.asarray(points[i])
+                        c = np.asarray(points[i + 1])
+                        ang = _angle(a, b, c)
+
+                        if _angle(a, b, c) <= 175:
+                            angles.append(ang)
+                        elif _angle(a, b, c) >= 185:
+                            angles.append(ang)
+                        else:
+                            continue
+                deviations = [abs(90 - i) for i in angles]
+                results_list.append(np.mean(deviations))
+
+            else:
+                results_list.append(np.nan)
 
         self.series = pd.Series(results_list, index=gdf.index)
 
 
 class EquivalentRectangularIndex:
     """
-    Calculates equivalent rectangular index of each object in given geoDataFrame.
+    Calculates equivalent rectangular index of each object in given GeoDataFrame.
 
     .. math::
         \\sqrt{{area} \\over \\textit{area of bounding rectangle}} * {\\textit{perimeter of bounding rectangle} \\over {perimeter}}
 
+    Based on :cite:`basaraner2017`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
     perimeters : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored perimeter value. If set to None, function will calculate perimeters
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored perimeter value. If set to ``None``, function will calculate perimeters
         during the process without saving them separately.
 
     Attributes
@@ -952,18 +931,9 @@ class EquivalentRectangularIndex:
     perimeters : Series
         Series containing used perimeter values
 
-    References
-    ----------
-    Basaraner M and Cetinkaya S (2017) Performance of shape indices and classification
-    schemes for characterising perceptual shape complexity of building footprints in GIS.
-    2nd ed. International Journal of Geographical Information Science, Taylor & Francis
-    31(10): 1952–1977. Available from:
-    https://www.tandfonline.com/doi/full/10.1080/13658816.2017.1346257.
-
     Examples
     --------
     >>> buildings_df['eri'] = momepy.EquivalentRectangularIndex(buildings_df, 'area', 'peri').series
-    100%|██████████| 144/144 [00:00<00:00, 895.57it/s]
     >>> buildings_df['eri'][0]
     0.7879229963118455
     """
@@ -1005,6 +975,8 @@ class Elongation:
 
     where `a` is the area of the object and `p` its perimeter.
 
+    Based on :cite:`gil2012`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
@@ -1017,12 +989,6 @@ class Elongation:
     gdf : GeoDataFrame
         original GeoDataFrame
 
-    References
-    ----------
-    Gil J, Montenegro N, Beirão JN, et al. (2012) On the Discovery of
-    Urban Typologies: Data Mining the Multi-dimensional Character of
-    Neighbourhoods. Urban Morphology 16(1): 27–40.
-
     Examples
     --------
     >>> buildings_df['elongation'] = momepy.Elongation(buildings_df).series
@@ -1063,10 +1029,14 @@ class CentroidCorners:
     .. math::
         \\overline{x}=\\frac{1}{n}\\left(\\sum_{i=1}^{n} dist_{i}\\right);\\space \\mathrm{SD}=\\sqrt{\\frac{\\sum|x-\\overline{x}|^{2}}{n}}
 
+    Adapted from :cite:`schirmer2015` and :cite:`cimburova2017`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -1077,12 +1047,6 @@ class CentroidCorners:
     gdf : GeoDataFrame
         original GeoDataFrame
 
-    References
-    ----------
-    Schirmer PM and Axhausen KW (2015) A multiscale classiﬁcation of urban morphology.
-    Journal of Transport and Land Use 9(1): 101–130.
-    + Cimburova (ADD)
-
     Examples
     --------
     >>> ccd = momepy.CentroidCorners(buildings_df)
@@ -1095,7 +1059,7 @@ class CentroidCorners:
     3.0810634305400177
     """
 
-    def __init__(self, gdf):
+    def __init__(self, gdf, verbose=True):
         self.gdf = gdf
         # define empty list for results
         results_list = []
@@ -1116,7 +1080,7 @@ class CentroidCorners:
             return False
 
         # iterating over rows one by one
-        for geom in tqdm(gdf.geometry, total=gdf.shape[0]):
+        for geom in tqdm(gdf.geometry, total=gdf.shape[0], disable=not verbose):
             distances = []  # set empty list of distances
             centroid = geom.centroid  # define centroid
             points = list(geom.exterior.coords)  # get points of a shape
@@ -1171,17 +1135,21 @@ class CentroidCorners:
 
 class Linearity:
     """
-    Calculates linearity of each LineString object in given geoDataFrame.
+    Calculates linearity of each LineString object in given GeoDataFrame.
 
     .. math::
         \\frac{l_{euclidean}}{l_{segment}}
 
     where `l` is the length of the LineString.
 
+    Adapted from :cite:`araldi2019`.
+
     Parameters
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing objects
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Attributes
     ----------
@@ -1190,38 +1158,28 @@ class Linearity:
     gdf : GeoDataFrame
         original GeoDataFrame
 
-    References
-    ----------
-    Araldi A and Fusco G (2017) Decomposing and Recomposing Urban Fabric:
-    The City from the Pedestrian Point of View. In:, pp. 365–376. Available
-    from: http://link.springer.com/10.1007/978-3-319-62407-5.
-
     Examples
     --------
     >>> streets_df['linearity'] = momepy.Linearity(streets_df).series
-    100%|██████████| 33/33 [00:00<00:00, 1737.64it/s]
     >>> streets_df['linearity'][0]
     1.0
     """
 
-    def __init__(self, gdf):
+    def __init__(self, gdf, verbose=True):
         self.gdf = gdf
-        # define empty list for results
-        results_list = []
 
-        lenghts = gdf.geometry.length
-        # fill new column with the value of area, iterating over rows one by one
-        # TODO use math instead of shapely points
-        for index, geom in tqdm(gdf.geometry.iteritems(), total=gdf.shape[0]):
-            euclidean = Point(geom.coords[0]).distance(Point(geom.coords[-1]))
-            results_list.append(euclidean / lenghts.loc[index])
+        euclidean = gdf.geometry.apply(
+            lambda geom: self._dist(geom.coords[0], geom.coords[-1])
+        )
+        self.series = euclidean / gdf.geometry.length
 
-        self.series = pd.Series(results_list, index=gdf.index)
+    def _dist(self, a, b):
+        return math.hypot(b[0] - a[0], b[1] - a[1])
 
 
 class CompactnessWeightedAxis:
     """
-    Calculates compactness-weighted axis of each object in given geoDataFrame.
+    Calculates compactness-weighted axis of each object in given GeoDataFrame.
 
     Initially designed for blocks.
 
@@ -1233,13 +1191,13 @@ class CompactnessWeightedAxis:
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored area value. If set to None, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
     perimeters : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored perimeter value. If set to None, function will calculate perimeters
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored perimeter value. If set to ``None``, function will calculate perimeters
         during the process without saving them separately.
     longest_axis : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, np.array, or pd.Series where is stored longest axis length value. If set to None, function will calculate it
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored longest axis length value. If set to ``None``, function will calculate it
         during the process without saving them separately.
 
     Attributes
