@@ -3,26 +3,30 @@
 
 import math
 import operator
+from distutils.version import LooseVersion
 
 import geopandas as gpd
 import libpysal
 import networkx as nx
 import numpy as np
+import pandas as pd
 import shapely
 from shapely.geometry import LineString, Point
 from tqdm import tqdm
 
 from .shape import CircularCompactness
 
+GPD_08 = str(gpd.__version__) >= LooseVersion("0.8.0")
+
 __all__ = [
     "unique_id",
-    "sw_high",
     "gdf_to_nx",
     "nx_to_gdf",
     "limit_range",
     "preprocess",
     "network_false_nodes",
     "snap_street_network_edge",
+    "CheckTessellationInput",
 ]
 
 
@@ -45,70 +49,6 @@ def unique_id(objects):
     return series
 
 
-def sw_high(k, gdf=None, weights=None, ids=None, contiguity="queen", silent=True):
-    """
-    Generate spatial weights based on Queen or Rook contiguity of order k.
-
-    Adjacent are all features within <= k steps. Pass either gdf or weights.
-    If both are passed, weights is used. If weights are passed, contiguity is
-    ignored and high order spatial weights based on `weights` are computed.
-
-    Parameters
-    ----------
-    k : int
-        order of contiguity
-    gdf : GeoDataFrame
-        GeoDataFrame containing objects to analyse. Index has to be consecutive range 0:x.
-        Otherwise, spatial weights will not match objects.
-    weights : libpysal.weights
-        libpysal.weights of order 1
-    contiguity : str (default 'queen')
-        type of contiguity weights. Can be 'queen' or 'rook'.
-    silent : bool (default True)
-        silence libpysal islands warnings
-
-    Returns
-    -------
-    libpysal.weights
-        libpysal.weights object
-
-    Examples
-    --------
-    >>> first_order = libpysal.weights.Queen.from_dataframe(geodataframe)
-    >>> first_order.mean_neighbors
-    5.848032564450475
-    >>> fourth_order = sw_high(k=4, gdf=geodataframe)
-    >>> fourth.mean_neighbors
-    85.73188602442333
-
-    """
-    if weights is not None:
-        first_order = weights
-    elif gdf is not None:
-        if contiguity == "queen":
-            first_order = libpysal.weights.Queen.from_dataframe(
-                gdf, ids=ids, silence_warnings=silent
-            )
-        elif contiguity == "rook":
-            first_order = libpysal.weights.Rook.from_dataframe(
-                gdf, ids=ids, silence_warnings=silent
-            )
-        else:
-            raise ValueError(
-                "{} is not supported. Use 'queen' or 'rook'.".format(contiguity)
-            )
-    else:
-        raise AttributeError("GeoDataFrame or spatial weights must be given.")
-
-    joined = first_order
-    for i in list(range(2, k + 1)):
-        i_order = libpysal.weights.higher_order(
-            first_order, k=i, silence_warnings=silent
-        )
-        joined = libpysal.weights.w_union(joined, i_order, silence_warnings=silent)
-    return joined
-
-
 def _angle(a, b, c):
     """
     Measure angle between a-b, b-c. In radians.
@@ -183,7 +123,7 @@ def gdf_to_nx(gdf_network, approach="primal", length="mm_len"):
     gdf_network : GeoDataFrame
         GeoDataFrame containing objects to convert
     approach : str, default 'primal'
-        Decide wheter genereate 'primal' or 'dual' graph.
+        Decide wheter genereate ``'primal'`` or ``'dual'`` graph.
     length : str, default mm_len
         name of attribute of segment length (geographical) which will be saved to graph
 
@@ -216,7 +156,7 @@ def gdf_to_nx(gdf_network, approach="primal", length="mm_len"):
     return net
 
 
-def _points_to_gdf(net, spatial_weights):
+def _points_to_gdf(net):
     """
     Generate point gdf from nodes.
     Helper for nx_to_gdf.
@@ -260,7 +200,7 @@ def _primal_to_gdf(net, points, lines, spatial_weights, nodeID):
     Helper for nx_to_gdf.
     """
     if points is True:
-        gdf_nodes = _points_to_gdf(net, spatial_weights)
+        gdf_nodes = _points_to_gdf(net)
 
         if spatial_weights is True:
             W = libpysal.weights.W.from_networkx(net)
@@ -293,12 +233,12 @@ def _dual_to_gdf(net):
 
 def nx_to_gdf(net, points=True, lines=True, spatial_weights=False, nodeID="nodeID"):
     """
-    Convert networkx.Graph to LineString GeoDataFrame and Point GeoDataFrame
+    Convert ``networkx.Graph`` to LineString GeoDataFrame and Point GeoDataFrame
 
     Parameters
     ----------
     net : networkx.Graph
-        networkx.Graph
+        ``networkx.Graph``
     points : bool
         export point-based gdf representing intersections
     lines : bool
@@ -315,30 +255,23 @@ def nx_to_gdf(net, points=True, lines=True, spatial_weights=False, nodeID="nodeI
 
     """
     # generate nodes and edges geodataframes from graph
+    primal = None
     if "approach" in net.graph.keys():
         if net.graph["approach"] == "primal":
-            nid = 1
-            for n in net:
-                net.nodes[n][nodeID] = nid
-                nid += 1
-            return _primal_to_gdf(
-                net,
-                points=points,
-                lines=lines,
-                spatial_weights=spatial_weights,
-                nodeID=nodeID,
-            )
-        if net.graph["approach"] == "dual":
+            primal = True
+        elif net.graph["approach"] == "dual":
             return _dual_to_gdf(net)
-        raise ValueError(
-            "Approach {} is not supported. Use 'primal' or 'dual'.".format(
-                net.graph["approach"]
+        else:
+            raise ValueError(
+                "Approach {} is not supported. Use 'primal' or 'dual'.".format(
+                    net.graph["approach"]
+                )
             )
-        )
 
-    import warnings
+    if not primal:
+        import warnings
 
-    warnings.warn("Approach is not set. Defaulting to 'primal'.")
+        warnings.warn("Approach is not set. Defaulting to 'primal'.")
 
     nid = 1
     for n in net:
@@ -374,7 +307,9 @@ def limit_range(vals, rng):
     return vals
 
 
-def preprocess(buildings, size=30, compactness=True, islands=True):
+def preprocess(
+    buildings, size=30, compactness=0.2, islands=True, loops=2, verbose=True
+):
     """
     Preprocesses building geometry to eliminate additional structures being single features.
 
@@ -388,9 +323,9 @@ def preprocess(buildings, size=30, compactness=True, islands=True):
     If feature area is smaller than set size it will be a) deleted if it does not
     touch any other feature; b) will be joined to feature with which it shares the
     longest boundary. If feature is fully within other feature, these will be joined.
-    If feature's circular compactness (:py:func:`momepy.circular_compactness`)
+    If feature's circular compactness (:py:class:`momepy.CircularCompactness`)
     is < 0.2, it will be joined to feature with which it shares the longest boundary.
-    Function does two loops through.
+    Function does multiple loops through.
 
 
     Parameters
@@ -400,12 +335,16 @@ def preprocess(buildings, size=30, compactness=True, islands=True):
     size : float (default 30)
         maximum area of feature to be considered as additional structure. Set to
         None if not wanted.
-    compactness : bool (default True)
-        if True, function will resolve additional structures identified based on
+    compactness : float (default .2)
+        if set, function will resolve additional structures identified based on
         their circular compactness.
     islands : bool (default True)
         if True, function will resolve additional structures which are fully within
         other structures (share 100% of exterior boundary).
+    loops : int (default 2)
+        number of loops
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Returns
     -------
@@ -415,8 +354,8 @@ def preprocess(buildings, size=30, compactness=True, islands=True):
     blg = buildings.copy()
     blg = blg.explode()
     blg.reset_index(drop=True, inplace=True)
-    for l in range(0, 2):
-        print("Loop", l + 1, "out of 2.")
+    for loop in range(0, loops):
+        print("Loop", loop + 1, f"out of {loops}.") if verbose else None
         blg.reset_index(inplace=True, drop=True)
         blg["mm_uid"] = range(len(blg))
         sw = libpysal.weights.contiguity.Rook.from_dataframe(blg, silence_warnings=True)
@@ -430,7 +369,10 @@ def preprocess(buildings, size=30, compactness=True, islands=True):
         delete = []
 
         for row in tqdm(
-            blg.itertuples(), total=blg.shape[0], desc="Identifying changes"
+            blg.itertuples(),
+            total=blg.shape[0],
+            desc="Identifying changes",
+            disable=not verbose,
         ):
             if size:
                 if row.geometry.area < size:
@@ -460,7 +402,7 @@ def preprocess(buildings, size=30, compactness=True, islands=True):
                     else:
                         delete.append(row.Index)
             if compactness:
-                if row.circu < 0.2:
+                if row.circu < compactness:
                     if row.n_count == 1:
                         uid = blg.iloc[row.neighbors[0]].mm_uid
                         if uid in join:
@@ -498,7 +440,9 @@ def preprocess(buildings, size=30, compactness=True, islands=True):
                         else:
                             join[uid] = [row.mm_uid]
 
-        for key in tqdm(join, total=len(join), desc="Changing geometry"):
+        for key in tqdm(
+            join, total=len(join), desc="Changing geometry", disable=not verbose
+        ):
             selection = blg[blg["mm_uid"] == key]
             if not selection.empty:
                 geoms = [selection.iloc[0].geometry]
@@ -515,20 +459,22 @@ def preprocess(buildings, size=30, compactness=True, islands=True):
     return blg[buildings.columns]
 
 
-def network_false_nodes(gdf, tolerance=0.1, precision=3):
+def network_false_nodes(gdf, tolerance=0.1, precision=3, verbose=True):
     """
     Check topology of street network and eliminate nodes of degree 2 by joining
-    affected edges. Attributes are not preserved.
+    affected edges.
 
     Parameters
     ----------
     gdf : GeoDataFrame, GeoSeries
-        GeoDataFrame  or GeoSeries containg edge representation of street network.
+        GeoDataFrame  or GeoSeries containing edge representation of street network.
     tolerance : float
-        nodes wihtin tolearance are seen as identical (floating point precision fix)
+        nodes within a tolerance are seen as identical (floating point precision fix)
     precision : int
         rounding parameter in estimating uniqueness of two points based on their
         coordinates
+    verbose : bool
+        if True, shows progress bars in loops
 
     Returns
     -------
@@ -538,77 +484,106 @@ def network_false_nodes(gdf, tolerance=0.1, precision=3):
         raise TypeError(
             "'gdf' should be GeoDataFrame or GeoSeries, got {}".format(type(gdf))
         )
-    streets = gdf.reset_index(drop=True).explode()
+    # double reset_index due to geopandas issue in explode
+    streets = gdf.reset_index(drop=True).explode().reset_index(drop=True)
     if isinstance(streets, gpd.GeoDataFrame):
         series = False
-        streets = streets.reset_index(drop=True).geometry
     elif isinstance(streets, gpd.GeoSeries):
-        streets = streets.reset_index(drop=True)
         series = True
 
     sindex = streets.sindex
 
     false_xy = []
-    print("Identifying false points...")
-    for idx, line in tqdm(streets.iteritems(), total=streets.shape[0]):
+    for line in tqdm(
+        streets.geometry,
+        total=streets.shape[0],
+        desc="Identifying false points",
+        disable=not verbose,
+    ):
         l_coords = list(line.coords)
         start = Point(l_coords[0]).buffer(tolerance)
         end = Point(l_coords[-1]).buffer(tolerance)
 
-        # find out whether ends of the line are connected or not
-        possible_first_index = list(sindex.intersection(start.bounds))
-        possible_first_matches = streets.iloc[possible_first_index]
-        possible_first_matches_clean = possible_first_matches.drop(idx, axis=0)
-        real_first_matches = possible_first_matches_clean[
-            possible_first_matches_clean.intersects(start)
-        ]
+        if GPD_08:
+            real_first_matches = sindex.query(start, predicate="intersects")
+            real_second_matches = sindex.query(end, predicate="intersects")
+        else:
+            # find out whether ends of the line are connected or not
+            possible_first_index = list(sindex.intersection(start.bounds))
+            possible_first_matches = streets.iloc[possible_first_index]
+            real_first_matches = possible_first_matches[
+                possible_first_matches.intersects(start)
+            ]
 
-        possible_second_index = list(sindex.intersection(end.bounds))
-        possible_second_matches = streets.iloc[possible_second_index]
-        possible_second_matches_clean = possible_second_matches.drop(idx, axis=0)
-        real_second_matches = possible_second_matches_clean[
-            possible_second_matches_clean.intersects(end)
-        ]
+            possible_second_index = list(sindex.intersection(end.bounds))
+            possible_second_matches = streets.iloc[possible_second_index]
+            real_second_matches = possible_second_matches[
+                possible_second_matches.intersects(end)
+            ]
 
-        if len(real_first_matches) == 1:
+        if len(real_first_matches) == 2:
             false_xy.append(
                 (round(l_coords[0][0], precision), round(l_coords[0][1], precision))
             )
-        if len(real_second_matches) == 1:
+        if len(real_second_matches) == 2:
             false_xy.append(
                 (round(l_coords[-1][0], precision), round(l_coords[-1][1], precision))
             )
 
-    false_unique = [Point(x) for x in set(false_xy)]
+    false_unique = list(set(false_xy))
+    x, y = zip(*false_unique)
+    if GPD_08:
+        points = gpd.points_from_xy(x, y).buffer(tolerance)
+    else:
+        points = gpd.GeoSeries(gpd.points_from_xy(x, y)).buffer(tolerance)
 
     geoms = streets
+    idx = max(geoms.index) + 1
+
+    for x, y, point in tqdm(
+        zip(x, y, points), desc="Merging segments", total=len(x), disable=not verbose
+    ):
+
+        if GPD_08:
+            predic = geoms.sindex.query(point, predicate="intersects")
+            matches = geoms.iloc[predic].geometry
+        else:
+            pos = list(geoms.sindex.intersection(point.bounds))
+            mat = geoms.iloc[pos]
+            matches = mat[mat.intersects(point)].geometry
 
-    print("Merging segments...")
-    for point in tqdm(false_unique):
-        matches = list(geoms[geoms.intersects(point.buffer(tolerance))].index)
-        idx = max(geoms.index) + 1
         try:
-            snap = shapely.ops.snap(geoms[matches[0]], geoms[matches[1]], tolerance)
-            multiline = snap.union(geoms[matches[1]])
+            snap = shapely.ops.snap(matches.iloc[0], matches.iloc[1], tolerance,)
+            multiline = snap.union(matches.iloc[1])
             linestring = shapely.ops.linemerge(multiline)
-            geoms = geoms.append(gpd.GeoSeries(linestring, index=[idx]))
-            geoms = geoms.drop(matches)
+            if linestring.type == "LineString":
+                if series:
+                    geoms.loc[idx] = linestring
+                else:
+                    geoms.loc[idx, "geometry"] = linestring
+                idx += 1
+            elif linestring.type == "MultiLineString":
+                for g in linestring.geoms:
+                    if series:
+                        geoms.loc[idx] = g
+                    else:
+                        geoms.loc[idx, "geometry"] = g
+                    idx += 1
+
+            geoms = geoms.drop(matches.index)
         except (IndexError, ValueError):
             import warnings
 
             warnings.warn(
                 "An exception during merging occured. "
-                "Lines at point [{x}, {y}] were not merged.".format(
-                    x=point.x, y=point.y
-                )
+                f"Lines at point [{x}, {y}] were not merged."
             )
 
-    geoms.crs = streets.crs
     streets = geoms.explode().reset_index(drop=True)
     if series:
+        streets.crs = gdf.crs
         return streets
-    geoms_gdf = gpd.GeoDataFrame(geometry=streets, crs=streets.crs)
-    return geoms_gdf
+    return streets
 
 
 def snap_street_network_edge(
@@ -618,11 +593,12 @@ def snap_street_network_edge(
     tessellation=None,
     tolerance_edge=None,
     edge=None,
+    verbose=True,
 ):
     """
-    Fix street network before performing blocks()
+    Fix street network before performing :class:`momepy.Blocks`.
 
-    Extends unjoined ends of street segments to join with other segmets or
+    Extends unjoined ends of street segments to join with other segments or
     tessellation boundary.
 
     Parameters
@@ -641,8 +617,10 @@ def snap_street_network_edge(
         tolerance in snapping to edge of tessellated area (by how much could be street
         segment extended).
     edge : Polygon
-        edge of area covered by morphological tessellation (same as `limit` in
-        :py:func:`momepy.tessellation`)
+        edge of area covered by morphological tessellation (same as ``limit`` in
+        :py:class:`momepy.Tessellation`)
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
 
     Returns
     -------
@@ -864,24 +842,28 @@ def snap_street_network_edge(
 
     network = edges.copy()
     # generating spatial index (rtree)
-    print("Building R-tree for network...")
+    print("Building spatial index for network...") if verbose else None
     sindex = network.sindex
-    print("Building R-tree for buildings...")
+    print("Building spatial index for buildings...") if verbose else None
     bindex = buildings.sindex
 
     def _get_geometry():
         if edge is not None:
             return edge.boundary
         if tessellation is not None:
-            print("Dissolving tesselation...")
+            print("Dissolving tesselation...") if verbose else None
             return tessellation.geometry.unary_union.boundary
         return None
 
     geometry = _get_geometry()
 
-    print("Snapping...")
     # iterating over each street segment
-    for idx, line in tqdm(network.geometry.iteritems(), total=network.shape[0]):
+    for idx, line in tqdm(
+        network.geometry.iteritems(),
+        total=network.shape[0],
+        desc="Snapping",
+        disable=not verbose,
+    ):
 
         l_coords = list(line.coords)
         # network_w = network.drop(idx, axis=0)['geometry']  # ensure that it wont intersect itself
@@ -923,7 +905,7 @@ def snap_street_network_edge(
                 if geometry is not None:
                     extend_line_edge(tolerance_edge, idx)
         else:
-            print("Something went wrong.")
+            print("Something went wrong.") if verbose else None
 
     return network
 
@@ -932,3 +914,122 @@ def _azimuth(point1, point2):
     """azimuth between 2 shapely points (interval 0 - 180)"""
     angle = np.arctan2(point2[0] - point1[0], point2[1] - point1[1])
     return np.degrees(angle) if angle > 0 else np.degrees(angle) + 180
+
+
+class CheckTessellationInput:
+    """
+    Check input data for :class:`Tessellation` for potential errors.
+
+    :class:`Tessellation` requires data of relatively high level of precision and there
+    are three particular patterns causing issues.\n
+    1. Features will collapse into empty polygon - these do not have tessellation
+    cell in the end.\n
+    2. Features will split into MultiPolygon - at some cases, features with narrow links
+    between parts split into two during 'shrinking'. In most cases that is not an issue
+    and resulting tessellation is correct anyway, but sometimes this result in a cell
+    being MultiPolygon, which is not correct.\n
+    3. Overlapping features - features which overlap even after 'shrinking' cause
+    invalid tessellation geometry.\n
+
+    :class:`CheckTessellationInput` will check for all of these. Overlapping features
+    have to be fixed prior Tessellation. Features which will split will cause issues
+    only sometimes, so
+    should be checked and fixed if necessary. Features which will collapse could
+    be
+    ignored, but they will have to excluded from next steps of tessellation-based analysis.
+
+    Parameters
+    ----------
+    gdf : GeoDataFrame or GeoSeries
+        GeoDataFrame containing objects to be used as ``gdf`` in :class:`Tessellation`
+    shrink : float (default 0.4)
+        distance for negative buffer
+    collapse : bool (default True)
+        check for features which would collapse to empty polygon
+    split : bool (default True)
+        check for features which would split into Multi-type
+    overlap : bool (default True)
+        check for overlapping features (after negative buffer)
+
+    Attributes
+    ----------
+    collapse : GeoDataFrame or GeoSeries
+        features which would collapse to empty polygon
+    split : GeoDataFrame or GeoSeries
+        features which would split into Multi-type
+    overlap : GeoDataFrame or GeoSeries
+        overlapping features (after negative buffer)
+
+
+    Examples
+    --------
+    >>> check = CheckTessellationData(df)
+    Collapsed features  : 3157
+    Split features      : 519
+    Overlapping features: 22
+    """
+
+    import warnings
+
+    warnings.filterwarnings("ignore", "GeoSeries.isna", UserWarning)
+
+    def __init__(self, gdf, shrink=0.4, collapse=True, split=True, overlap=True):
+        data = gdf[~gdf.is_empty]
+
+        if split:
+            types = data.type
+
+        if shrink != 0:
+            shrink = data.buffer(-shrink)
+        else:
+            shrink = data
+
+        if collapse:
+            emptycheck = shrink.is_empty
+            self.collapse = gdf[emptycheck]
+            collapsed = len(self.collapse)
+        else:
+            collapsed = "NA"
+
+        if split:
+            type_check = shrink.type != types
+            self.split = gdf[type_check]
+            split_count = len(self.split)
+        else:
+            split_count = "NA"
+
+        if overlap:
+            shrink = shrink.reset_index(drop=True)
+            shrink = shrink[~(shrink.is_empty | shrink.geometry.isna())]
+            sindex = shrink.sindex
+            hits = shrink.bounds.apply(
+                lambda row: list(sindex.intersection(row)), axis=1
+            )
+            od_matrix = pd.DataFrame(
+                {
+                    "origin": np.repeat(hits.index, hits.apply(len)),
+                    "dest": np.concatenate(hits.values),
+                }
+            )
+            od_matrix = od_matrix[od_matrix.origin != od_matrix.dest]
+            duplicated = pd.DataFrame(np.sort(od_matrix, axis=1)).duplicated()
+            od_matrix = od_matrix.reset_index(drop=True)[~duplicated]
+            od_matrix = od_matrix.join(
+                shrink.geometry.rename("o_geom"), on="origin"
+            ).join(shrink.geometry.rename("d_geom"), on="dest")
+            intersection = od_matrix.o_geom.values.intersection(od_matrix.d_geom.values)
+            type_filter = gpd.GeoSeries(intersection).type == "Polygon"
+            empty_filter = intersection.is_empty
+            overlapping = od_matrix.reset_index(drop=True)[empty_filter ^ type_filter]
+            over_rows = sorted(overlapping.origin.append(overlapping.dest).unique())
+
+            self.overlap = gdf.iloc[over_rows]
+            overlapping_c = len(self.overlap)
+        else:
+            overlapping_c = "NA"
+
+        print(
+            "Collapsed features  : {0}\n"
+            "Split features      : {1}\n"
+            "Overlapping features: {2}".format(collapsed, split_count, overlapping_c)
+        )
